@@ -12,9 +12,17 @@ from debputy.build_support.build_logic import (
 )
 from debputy.build_support.buildsystem_detection import auto_detect_buildsystem
 from debputy.commands.debputy_cmd.context import CommandContext
+from debputy.filesystem_scan import FSROOverlay
 from debputy.highlevel_manifest import HighLevelManifest
 from debputy.plugins.debputy.to_be_api_types import BuildSystemRule, CleanHelper
-from debputy.util import _info, print_command, _error, _debug_log, _warn
+from debputy.util import (
+    _info,
+    print_command,
+    _error,
+    _debug_log,
+    _warn,
+    PRINT_BUILD_SYSTEM_COMMAND,
+)
 from debputy.util import (
     run_build_system_command,
 )
@@ -67,8 +75,8 @@ def _debhelper_left_overs() -> bool:
 class CleanHelperImpl(CleanHelper):
 
     def __init__(self) -> None:
-        self.files_to_remove: Set[str] = set()
-        self.dirs_to_remove: Set[str] = set()
+        self.files_to_remove: set[str] = set()
+        self.dirs_to_remove: set[str] = set()
 
     def schedule_removal_of_files(self, *args: str) -> None:
         self.files_to_remove.update(args)
@@ -104,6 +112,31 @@ def _scan_for_standard_removals(clean_helper: CleanHelperImpl) -> None:
                 remove_files.add(path)
 
 
+def _apply_remove_during_clean_rules(
+    clean_helper: CleanHelper,
+    manifest: HighLevelManifest,
+) -> None:
+    source_root = FSROOverlay.create_root_dir(".", ".")
+    had_errors = False
+    for rule in manifest.remove_during_clean_rules:
+        allow_dir_matches = rule.raw_match_rule.endswith("/")
+        for match in rule.match_rule.finditer(source_root):
+            if match.is_dir and not allow_dir_matches:
+                _warn(
+                    f' * The path {match.path} is a directory and the remove-during-clean rule matching it does not explicitly end with a "/". Please add a slash to {rule.raw_match_rule} (at {rule.attribute_path.path}) if it is intended to remove directories'
+                )
+                had_errors = True
+                continue
+
+            if match.is_dir:
+                clean_helper.schedule_removal_of_directories(match.path)
+            else:
+                clean_helper.schedule_removal_of_files(match.path)
+
+    if had_errors:
+        _error("Aborting during to the above errors")
+
+
 def perform_clean(
     context: CommandContext,
     manifest: HighLevelManifest,
@@ -178,16 +211,19 @@ def perform_clean(
         debhelper_used = True
 
     _scan_for_standard_removals(clean_helper)
+    _apply_remove_during_clean_rules(clean_helper, manifest)
 
     for package in manifest.all_packages:
         package_staging_dir = os.path.join("debian", package.name)
         if os.path.lexists(package_staging_dir):
             clean_helper.schedule_removal_of_directories(package_staging_dir)
 
-    remove_files = clean_helper.files_to_remove
-    remove_dirs = clean_helper.dirs_to_remove
+    remove_files = sorted(clean_helper.files_to_remove)
+    remove_dirs = sorted(clean_helper.dirs_to_remove)
     if remove_files:
-        print_command("rm", "-f", *remove_files)
+        print_command(
+            "rm", "-f", *remove_files, print_at_log_level=PRINT_BUILD_SYSTEM_COMMAND
+        )
         _remove_files_if_exists(*remove_files)
     if remove_dirs:
         run_build_system_command("rm", "-fr", *remove_dirs)
