@@ -18,13 +18,12 @@ from typing import (
     NamedTuple,
     Literal,
     cast,
-    Sequence,
     FrozenSet,
 )
+from collections.abc import Sequence
 
 from debputy import DEBPUTY_ROOT_DIR, DEBPUTY_PLUGIN_ROOT_DIR
 from debputy.analysis import REFERENCE_DATA_TABLE
-from debputy.analysis.debian_dir import scan_debian_dir
 from debputy.build_support import perform_clean, perform_builds
 from debputy.commands.debputy_cmd.context import (
     CommandContext,
@@ -118,8 +117,8 @@ class SharedArgument(NamedTuple):
     """
 
     action: argparse.Action
-    args: Tuple[Any, ...]
-    kwargs: Dict[str, Any]
+    args: tuple[Any, ...]
+    kwargs: dict[str, Any]
 
 
 class Namespace(argparse.Namespace):
@@ -430,7 +429,7 @@ _DH_PLUGIN_PKG_DIR = "debputy-plugins"
 
 def _find_plugins_and_tests_in_source_package(
     context: CommandContext,
-) -> Tuple[bool, List[Tuple[DebputyPluginMetadata, str]], List[str]]:
+) -> tuple[bool, list[tuple[DebputyPluginMetadata, str]], list[str]]:
     debian_dir = context.debian_dir
     binary_packages = context.binary_packages()
     installs = []
@@ -564,13 +563,13 @@ def _dh_integration_install_plugin(context: CommandContext) -> None:
 
 def _run_tests(
     context: CommandContext,
-    test_paths: List[str],
+    test_paths: list[str],
     *,
-    cwd: Optional[str] = None,
-    tmpdir_root: Optional[str] = None,
+    cwd: str | None = None,
+    tmpdir_root: str | None = None,
     test_plugin_location: Literal["installed", "uninstalled"] = "uninstalled",
-    on_error_return: Optional[Any] = None,
-    on_success_return: Optional[Any] = True,
+    on_error_return: Any | None = None,
+    on_success_return: Any | None = True,
 ) -> Any:
     env = dict(os.environ)
     env["DEBPUTY_TEST_PLUGIN_LOCATION"] = test_plugin_location
@@ -811,7 +810,7 @@ def _dpkg_build_driver_integration(context: CommandContext) -> None:
     elif task_name in ("build", "build-indep", "build-arch"):
         perform_builds(context, manifest)
     elif task_name in ("binary", "binary-indep", "binary-arch"):
-        build_system_install_dirs: List[Tuple[str, FrozenSet[BinaryPackage]]] = []
+        build_system_install_dirs: list[tuple[str, frozenset[BinaryPackage]]] = []
         perform_builds(context, manifest, build_system_install_dirs)
         assemble(
             context,
@@ -879,7 +878,7 @@ def assemble(
     context: CommandContext,
     manifest: HighLevelManifest,
     integration_mode: DebputyIntegrationMode,
-    build_system_install_dirs: Sequence[Tuple[str, FrozenSet[BinaryPackage]]],
+    build_system_install_dirs: Sequence[tuple[str, frozenset[BinaryPackage]]],
     *,
     debug_materialization: bool = False,
 ) -> None:
@@ -1034,6 +1033,8 @@ def _annotate_debian_directory(context: CommandContext) -> None:
     binary_packages = context.binary_packages()
     feature_set = context.load_plugins()
 
+    from debputy.analysis.debian_dir import scan_debian_dir
+
     result = scan_debian_dir(
         feature_set,
         context.source_package(),
@@ -1258,8 +1259,8 @@ def _error_w_stack_trace(
     error_msg: str,
     stacktrace: BaseException,
     debug_mode: bool,
-    orig_exception: Optional[BaseException] = None,
-    follow_warning: Optional[List[str]] = None,
+    orig_exception: BaseException | None = None,
+    follow_warning: list[str] | None = None,
 ) -> "NoReturn":
     if debug_mode:
         _warn(
