@@ -6,7 +6,7 @@ from argparse import BooleanOptionalAction
 from typing import TYPE_CHECKING
 
 from debputy.commands.debputy_cmd.context import ROOT_COMMAND, CommandContext, add_arg
-from debputy.lsp.lsp_reference_keyword import ALL_PUBLIC_NAMED_STYLES
+from debputy.lsp.named_styles import ALL_PUBLIC_NAMED_STYLES
 from debputy.util import _error, _info
 
 if TYPE_CHECKING:
