@@ -2,18 +2,16 @@ import json
 import os
 import re
 import subprocess
+import sys
 from itertools import chain
 from typing import (
     Optional,
     List,
-    Callable,
     Set,
-    Mapping,
     FrozenSet,
     Tuple,
-    Iterable,
-    Container,
 )
+from collections.abc import Callable, Mapping, Iterable, Container
 
 from debian.deb822 import Deb822
 
@@ -39,7 +37,7 @@ from debputy.plugin.api import VirtualPath
 from debputy.plugin.api.spec import DebputyIntegrationMode, INTEGRATION_MODE_FULL
 from debputy.util import _error, _warn, _info, escape_shell, assume_not_none
 
-SUPPORTED_MIGRATIONS: Mapping[MigrationTarget, FrozenSet[MigrationTarget]] = {
+SUPPORTED_MIGRATIONS: Mapping[MigrationTarget, frozenset[MigrationTarget]] = {
     INTEGRATION_MODE_FULL: frozenset([INTEGRATION_MODE_FULL]),
     INTEGRATION_MODE_DH_DEBPUTY: frozenset(
         [
@@ -63,11 +61,11 @@ SUPPORTED_MIGRATIONS: Mapping[MigrationTarget, FrozenSet[MigrationTarget]] = {
 
 def _print_migration_summary(
     fo: IOBasedOutputStyling,
-    migrations: List[FeatureMigration],
+    migrations: list[FeatureMigration],
     compat: int,
     min_compat_level: int,
-    required_plugins: Set[str],
-    requested_plugins: Optional[Set[str]],
+    required_plugins: set[str],
+    requested_plugins: set[str] | None,
 ) -> None:
     warning_count = 0
 
@@ -135,7 +133,7 @@ def _print_migration_summary(
         )
 
 
-def _dh_compat_level() -> Optional[int]:
+def _dh_compat_level() -> int | None:
     try:
         res = subprocess.check_output(
             ["dh_assistant", "active-compat-level"], stderr=subprocess.DEVNULL
@@ -153,7 +151,7 @@ def _dh_compat_level() -> Optional[int]:
     return compat
 
 
-def _requested_debputy_plugins(debian_dir: VirtualPath) -> Optional[Set[str]]:
+def _requested_debputy_plugins(debian_dir: VirtualPath) -> set[str] | None:
     ctrl_file = debian_dir.get("control")
     if not ctrl_file:
         return None
@@ -183,7 +181,7 @@ def _requested_debputy_plugins(debian_dir: VirtualPath) -> Optional[Set[str]]:
 
 def _check_migration_target(
     context: CommandContext,
-    migration_target: Optional[DebputyIntegrationMode],
+    migration_target: DebputyIntegrationMode | None,
 ) -> DebputyIntegrationMode:
     r = read_dh_addon_sequences(context.debian_dir)
     if r is not None:
@@ -230,7 +228,7 @@ def _check_migration_target(
 
 def _read_git_status_lines(
     lines: Iterable[bytes],
-) -> Iterable[Tuple[str, str, Optional[str]]]:
+) -> Iterable[tuple[str, str, str | None]]:
     line_iter = iter(lines)
     while True:
         try:
@@ -262,7 +260,7 @@ def _read_git_status_lines(
         yield status_marker, filename, second_filename
 
 
-def _git_status() -> Tuple[bool, Container[str]]:
+def _git_status() -> tuple[bool, Container[str]]:
     try:
         top_level = (
             subprocess.check_output(
@@ -313,7 +311,7 @@ def _git_status() -> Tuple[bool, Container[str]]:
 
 def _check_vcs_clashes(
     manifest: HighLevelManifest,
-    migrations: List[FeatureMigration],
+    migrations: list[FeatureMigration],
     untracked_files: Container[str],
 ) -> None:
     all_affected_files = []
@@ -331,7 +329,7 @@ def _check_vcs_clashes(
     clashing_paths = [p for p in all_affected_files if p in untracked_files]
 
     if clashing_paths:
-        print()
+        print(file=sys.stderr)
         _warn(
             "The following untracked or ignored paths would be affected by this migration"
         )
@@ -341,7 +339,7 @@ def _check_vcs_clashes(
         _warn("")
         _warn("Since they are not tracked, the VCS cannot undo the changes to them")
         _warn("if the migration continued")
-        print()
+        print(file=sys.stderr)
         _error(
             "Please stash, commit or remove the above files before continuing. Alternatively, use --ignore-vcs"
         )
@@ -351,10 +349,10 @@ def perform_migration(
     fo: IOBasedOutputStyling,
     manifest: HighLevelManifest,
     acceptable_migration_issues: AcceptableMigrationIssues,
-    permit_destructive_changes: Optional[bool],
+    permit_destructive_changes: bool | None,
     migration_target: DebputyIntegrationMode,
     manifest_parser_factory: Callable[[str], HighLevelManifest],
-    migrators: List[Migrator],
+    migrators: list[Migrator],
     *,
     ignore_vcs: bool = False,
 ) -> None:
@@ -420,12 +418,9 @@ def perform_migration(
             "\n\nPlease review the situation and resolve the conflict manually."
         )
 
-    # We start on compat 12 for arch:any due to the new dh_makeshlibs and dh_installinit default
-    # For arch:any, the min is compat 14 due to `dh_dwz` being removed.
-    min_compat = 14 if any(not p.is_arch_all for p in manifest.all_packages) else 12
     min_compat = max(
         (m.assumed_compat for m in migrations if m.assumed_compat is not None),
-        default=min_compat,
+        default=0,
     )
 
     if compat < min_compat and "min-compat-level" not in acceptable_migration_issues:
@@ -438,7 +433,7 @@ def perform_migration(
         )
 
     requested_plugins = _requested_debputy_plugins(debian_dir)
-    required_plugins: Set[str] = set()
+    required_plugins: set[str] = set()
     required_plugins.update(
         chain.from_iterable(
             m.required_plugins for m in migrations if m.required_plugins
@@ -540,7 +535,7 @@ def perform_migration(
 
 
 def _apply_renames_and_removals(
-    migrations: List[FeatureMigration],
+    migrations: list[FeatureMigration],
     uses_git: bool,
 ) -> None:
     for previous_path, new_path in (
