@@ -5,6 +5,7 @@ import errno
 import io
 import operator
 import os
+import shutil
 import stat
 import subprocess
 import tempfile
@@ -13,13 +14,10 @@ from abc import ABC
 from contextlib import suppress
 from typing import (
     List,
-    Iterable,
     Dict,
     Optional,
     Tuple,
     Union,
-    Iterator,
-    Mapping,
     cast,
     Any,
     ContextManager,
@@ -28,11 +26,11 @@ from typing import (
     NoReturn,
     Type,
     Generic,
-    Callable,
     TypeVar,
     overload,
     Literal,
 )
+from collections.abc import Iterable, Iterator, Mapping, Callable
 from weakref import ref, ReferenceType
 
 from debputy.exceptions import (
@@ -102,7 +100,7 @@ class AlwaysEmptyReadOnlyMetadataReference(PathMetadataReference[PMT]):
         self,
         owning_plugin: str,
         current_plugin: str,
-        metadata_type: Type[PMT],
+        metadata_type: type[PMT],
     ) -> None:
         self._owning_plugin = owning_plugin
         self._current_plugin = current_plugin
@@ -121,7 +119,7 @@ class AlwaysEmptyReadOnlyMetadataReference(PathMetadataReference[PMT]):
         return False
 
     @property
-    def value(self) -> Optional[PMT]:
+    def value(self) -> PMT | None:
         if self.can_read:
             return None
         raise DebputyMetadataAccessError(
@@ -150,8 +148,8 @@ class AlwaysEmptyReadOnlyMetadataReference(PathMetadataReference[PMT]):
 @dataclasses.dataclass(slots=True)
 class PathMetadataValue(Generic[PMT]):
     owning_plugin: str
-    metadata_type: Type[PMT]
-    value: Optional[PMT] = None
+    metadata_type: type[PMT]
+    value: PMT | None = None
 
     def can_read_value(self, current_plugin: str) -> bool:
         return self.owning_plugin == current_plugin
@@ -191,7 +189,7 @@ class PathMetadataReferenceImplementation(PathMetadataReference[PMT]):
         return owning_path.is_read_write and not owning_path.is_detached
 
     @property
-    def value(self) -> Optional[PMT]:
+    def value(self) -> PMT | None:
         if self.can_read:
             return self._path_metadata_value.value
         raise DebputyMetadataAccessError(
@@ -245,7 +243,7 @@ def _cp_a(source: str, dest: str) -> None:
         )
 
 
-def _split_path(path: str) -> Tuple[bool, bool, List[str]]:
+def _split_path(path: str) -> tuple[bool, bool, list[str]]:
     must_be_dir = True if path.endswith("/") else False
     absolute = False
     if path.startswith("/"):
@@ -327,7 +325,7 @@ class VirtualPathBase(VirtualPath, ABC):
             return None
         return match
 
-    def attempt_lookup(self, path: str) -> Tuple["VirtualPathBase", List[str]]:
+    def attempt_lookup(self, path: str) -> tuple["VirtualPathBase", list[str]]:
         if self.is_detached:
             raise ValueError(
                 f'Cannot perform lookup via "{self._orphan_safe_path()}": The path is detached'
@@ -450,7 +448,7 @@ class VirtualPathBase(VirtualPath, ABC):
         name: str,
         mode: OpenMode = "r",
         buffering: int = -1,
-    ) -> Union[TextIO, BinaryIO]:
+    ) -> TextIO | BinaryIO:
         """Open a child path of the current directory in a given mode. Usually used with a context manager
 
         The path is opened according to the `mode` parameter similar to the built-in `open` in Python.
@@ -529,26 +527,26 @@ class FSPath(VirtualPathBase, ABC):
         self,
         basename: str,
         parent: Optional["FSPath"],
-        children: Optional[Dict[str, "FSPath"]] = None,
-        initial_mode: Optional[int] = None,
-        mtime: Optional[float] = None,
-        stat_cache: Optional[os.stat_result] = None,
+        children: dict[str, "FSPath"] | None = None,
+        initial_mode: int | None = None,
+        mtime: float | None = None,
+        stat_cache: os.stat_result | None = None,
     ) -> None:
         self._basename = basename
-        self._path_cache: Optional[str] = None
-        self._parent_path_cache: Optional[str] = None
+        self._path_cache: str | None = None
+        self._parent_path_cache: str | None = None
         self._children = children
-        self._last_known_parent_path: Optional[str] = None
+        self._last_known_parent_path: str | None = None
         self._mode = initial_mode
         self._mtime = mtime
         self._stat_cache = stat_cache
-        self._metadata: Dict[Tuple[str, Type[Any]], PathMetadataValue[Any]] = {}
+        self._metadata: dict[tuple[str, type[Any]], PathMetadataValue[Any]] = {}
         self._owner = ROOT_DEFINITION
         self._group = ROOT_DEFINITION
 
         # The self._parent_dir = None is to create `_parent_dir` because the parent_dir setter calls
         # is_orphaned, which assumes self._parent_dir is an attribute.
-        self._parent_dir: Optional[ReferenceType["FSPath"]] = None
+        self._parent_dir: ReferenceType["FSPath"] | None = None
         if parent is not None:
             self.parent_dir = parent
 
@@ -598,7 +596,7 @@ class FSPath(VirtualPathBase, ABC):
             if current.is_dir and not current.is_detached:
                 stack.extend(sorted(current.iterdir, key=by_basename, reverse=True))
 
-    def walk(self) -> Iterable[Tuple["FSPath", List["FSPath"]]]:
+    def walk(self) -> Iterable[tuple["FSPath", list["FSPath"]]]:
         # FIXME: can this be more "os.walk"-like without making it harder to implement?
         if not self.is_dir:
             yield self, []
@@ -737,15 +735,15 @@ class FSPath(VirtualPathBase, ABC):
         self._parent_path_cache = None
 
     @property
-    def parent_dir_path(self) -> Optional[str]:
+    def parent_dir_path(self) -> str | None:
         if self.is_detached:
             return self._last_known_parent_path
         return assume_not_none(self.parent_dir).path
 
     def chown(
         self,
-        owner: Optional[StaticFileSystemOwner],
-        group: Optional[StaticFileSystemGroup],
+        owner: StaticFileSystemOwner | None,
+        group: StaticFileSystemGroup | None,
     ) -> None:
         """Change the owner/group of this path
 
@@ -822,7 +820,7 @@ class FSPath(VirtualPathBase, ABC):
         self._mtime = new_mtime
 
     @property
-    def tar_owner_info(self) -> Tuple[str, int, str, int]:
+    def tar_owner_info(self) -> tuple[str, int, str, int]:
         owner = self._owner
         group = self._group
         return (
@@ -844,10 +842,10 @@ class FSPath(VirtualPathBase, ABC):
         unlink_if_exists: bool = True,
         use_fs_path_mode: bool = False,
         mode: int = 0o0644,
-        mtime: Optional[float] = None,
+        mtime: float | None = None,
         # Special-case parameters that are not exposed in the API
         fs_basename_matters: bool = False,
-        subdir_key: Optional[str] = None,
+        subdir_key: str | None = None,
     ) -> Iterator["FSPath"]:
         if "/" in name or name in {".", ".."}:
             raise ValueError(f'Invalid file name: "{name}"')
@@ -919,7 +917,7 @@ class FSPath(VirtualPathBase, ABC):
         mode: int = 0o0644,
         require_copy_on_write: bool = True,
         follow_symlinks: bool = True,
-        reference_path: Optional[VirtualPath] = None,
+        reference_path: VirtualPath | None = None,
     ) -> "FSPath":
         if "/" in name or name in {".", ".."}:
             raise ValueError(f'Invalid file name: "{name}"')
@@ -942,7 +940,7 @@ class FSPath(VirtualPathBase, ABC):
                 )
             new_fs_path = os.path.realpath(new_fs_path, strict=True)
 
-        fmode: Optional[int] = mode
+        fmode: int | None = mode
         if use_fs_path_mode:
             fmode = None
 
@@ -976,7 +974,7 @@ class FSPath(VirtualPathBase, ABC):
         link_name: str,
         link_target: str,
         *,
-        reference_path: Optional[VirtualPath] = None,
+        reference_path: VirtualPath | None = None,
     ) -> "FSPath":
         if "/" in link_name or link_name in {".", ".."}:
             raise ValueError(
@@ -1005,7 +1003,7 @@ class FSPath(VirtualPathBase, ABC):
         self,
         name: str,
         *,
-        reference_path: Optional[VirtualPath] = None,
+        reference_path: VirtualPath | None = None,
     ) -> "FSPath":
         if "/" in name or name in {".", ".."}:
             raise ValueError(
@@ -1065,9 +1063,9 @@ class FSPath(VirtualPathBase, ABC):
 
     def metadata(
         self,
-        metadata_type: Type[PMT],
+        metadata_type: type[PMT],
         *,
-        owning_plugin: Optional[str] = None,
+        owning_plugin: str | None = None,
     ) -> PathMetadataReference[PMT]:
         current_plugin = self._current_plugin()
         if owning_plugin is None:
@@ -1139,10 +1137,10 @@ class VirtualFSPathBase(FSPath, ABC):
         self,
         basename: str,
         parent: Optional["FSPath"],
-        children: Optional[Dict[str, "FSPath"]] = None,
-        initial_mode: Optional[int] = None,
-        mtime: Optional[float] = None,
-        stat_cache: Optional[os.stat_result] = None,
+        children: dict[str, "FSPath"] | None = None,
+        initial_mode: int | None = None,
+        mtime: float | None = None,
+        stat_cache: os.stat_result | None = None,
     ) -> None:
         super().__init__(
             basename,
@@ -1186,7 +1184,7 @@ class VirtualFSPathBase(FSPath, ABC):
 class FSRootDir(FSPath):
     __slots__ = ("_fs_path", "_fs_read_write", "_plugin_context")
 
-    def __init__(self, fs_path: Optional[str] = None) -> None:
+    def __init__(self, fs_path: str | None = None) -> None:
         self._fs_path = fs_path
         self._fs_read_write = True
         super().__init__(
@@ -1213,12 +1211,12 @@ class FSRootDir(FSPath):
         return None
 
     @parent_dir.setter
-    def parent_dir(self, new_parent: Optional[FSPath]) -> None:
+    def parent_dir(self, new_parent: FSPath | None) -> None:
         if new_parent is not None:
             raise ValueError("The root directory cannot become a non-root directory")
 
     @property
-    def parent_dir_path(self) -> Optional[str]:
+    def parent_dir_path(self) -> str | None:
         return None
 
     @property
@@ -1293,7 +1291,7 @@ class VirtualPathWithReference(VirtualFSPathBase, ABC):
         parent: FSPath,
         *,
         default_mode: int,
-        reference_path: Optional[VirtualPath] = None,
+        reference_path: VirtualPath | None = None,
     ) -> None:
         super().__init__(
             basename,
@@ -1346,7 +1344,7 @@ class VirtualPathWithReference(VirtualFSPathBase, ABC):
         *,
         byte_io: bool = False,
         buffering: int = -1,
-    ) -> Union[TextIO, BinaryIO]:
+    ) -> TextIO | BinaryIO:
         reference_path = self._reference_path
         if reference_path is not None and reference_path.fs_path == self.fs_path:
             return reference_path.open(byte_io=byte_io, buffering=buffering)
@@ -1361,7 +1359,7 @@ class VirtualDirectoryFSPath(VirtualPathWithReference):
         basename: str,
         parent: FSPath,
         *,
-        reference_path: Optional[VirtualPath] = None,
+        reference_path: VirtualPath | None = None,
     ) -> None:
         super().__init__(
             basename,
@@ -1398,7 +1396,7 @@ class SymlinkVirtualPath(VirtualPathWithReference):
         parent_dir: FSPath,
         link_target: str,
         *,
-        reference_path: Optional[VirtualPath] = None,
+        reference_path: VirtualPath | None = None,
     ) -> None:
         super().__init__(
             basename,
@@ -1438,10 +1436,10 @@ class FSBackedFilePath(VirtualPathWithReference):
         fs_path: str,
         *,
         replaceable_inline: bool = False,
-        initial_mode: Optional[int] = None,
-        mtime: Optional[float] = None,
-        stat_cache: Optional[os.stat_result] = None,
-        reference_path: Optional[VirtualPath] = None,
+        initial_mode: int | None = None,
+        mtime: float | None = None,
+        stat_cache: os.stat_result | None = None,
+        reference_path: VirtualPath | None = None,
     ) -> None:
         super().__init__(
             basename,
@@ -1510,15 +1508,15 @@ class VirtualTestPath(FSPath):
     def __init__(
         self,
         basename: str,
-        parent_dir: Optional[FSPath],
-        mode: Optional[int] = None,
-        mtime: Optional[float] = None,
+        parent_dir: FSPath | None,
+        mode: int | None = None,
+        mtime: float | None = None,
         is_dir: bool = False,
-        has_fs_path: Optional[bool] = False,
-        fs_path: Optional[str] = None,
-        link_target: Optional[str] = None,
-        content: Optional[str] = None,
-        materialized_content: Optional[str] = None,
+        has_fs_path: bool | None = False,
+        fs_path: str | None = None,
+        link_target: str | None = None,
+        content: str | None = None,
+        materialized_content: str | None = None,
     ) -> None:
         if is_dir:
             self._path_type = PathType.DIRECTORY
@@ -1662,7 +1660,7 @@ class VirtualTestPath(FSPath):
         name: str,
         mode: OpenMode = "r",
         buffering: int = -1,
-    ) -> Union[TextIO, BinaryIO]:
+    ) -> TextIO | BinaryIO:
         existing = self.get(name)
         if existing or "r" in mode:
             with super().open_child(name, mode, buffering=buffering) as fd:
@@ -1689,13 +1687,13 @@ class VirtualTestPath(FSPath):
         *,
         byte_io: bool = False,
         buffering: int = -1,
-    ) -> Union[TextIO, BinaryIO]:
+    ) -> TextIO | BinaryIO:
         if self._content is None:
             try:
                 return super().open(byte_io=byte_io, buffering=buffering)
             except FileNotFoundError as e:
                 raise TestPathWithNonExistentFSPathError(
-                    "The test path {self.path} had an fs_path {self._fs_path}, which does not"
+                    f"The test path {self.path} had an fs_path {self._fs_path}, which does not"
                     " exist.  This exception can only occur in the testsuite.  Either have the"
                     " test provide content for the path (`virtual_path_def(..., content=...) or,"
                     " if that is too painful in general, have the code accept this error as a "
@@ -1722,12 +1720,12 @@ class FSOverlayBase(VirtualPathBase, Generic[FSP]):
         self,
         path: str,
         fs_path: str,
-        parent: Optional[FSP],
+        parent: FSP | None,
     ) -> None:
         self._path: str = path
         prefix = "/" if fs_path.startswith("/") else ""
         self._fs_path: str = prefix + _normalize_path(fs_path, with_prefix=False)
-        self._parent: Optional[ReferenceType[FSP]] = (
+        self._parent: ReferenceType[FSP] | None = (
             ref(parent) if parent is not None else None
         )
 
@@ -1791,7 +1789,7 @@ class FSOverlayBase(VirtualPathBase, Generic[FSP]):
         *,
         byte_io: bool = False,
         buffering: int = -1,
-    ) -> Union[TextIO, BinaryIO]:
+    ) -> TextIO | BinaryIO:
         # Allow symlinks for open here, because we can let the OS resolve the symlink reliably in this
         # case.
         if not self.is_file and not self.is_symlink:
@@ -1801,13 +1799,13 @@ class FSOverlayBase(VirtualPathBase, Generic[FSP]):
 
         if byte_io:
             return open(self.fs_path, "rb", buffering=buffering)
-        return open(self.fs_path, "rt", encoding="utf-8", buffering=buffering)
+        return open(self.fs_path, encoding="utf-8", buffering=buffering)
 
     def metadata(
         self,
-        metadata_type: Type[PMT],
+        metadata_type: type[PMT],
         *,
-        owning_plugin: Optional[str] = None,
+        owning_plugin: str | None = None,
     ) -> PathMetadataReference[PMT]:
         current_plugin = self._current_plugin()
         if owning_plugin is None:
@@ -1866,10 +1864,10 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
         parent: Optional["FSROOverlay"],
     ) -> None:
         super().__init__(path, fs_path, parent=parent)
-        self._stat_cache: Optional[os.stat_result] = None
-        self._readlink_cache: Optional[str] = None
+        self._stat_cache: os.stat_result | None = None
+        self._readlink_cache: str | None = None
         self._stat_failed_cache = False
-        self._children: Optional[Mapping[str, FSROOverlay]] = None
+        self._children: Mapping[str, FSROOverlay] | None = None
 
     @classmethod
     def create_root_dir(cls, path: str, fs_path: str) -> "FSROOverlay":
@@ -1981,8 +1979,8 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
 
     def chown(
         self,
-        owner: Optional[StaticFileSystemOwner],
-        group: Optional[StaticFileSystemGroup],
+        owner: StaticFileSystemOwner | None,
+        group: StaticFileSystemGroup | None,
     ) -> None:
         self._error_ro_fs()
 
@@ -1996,7 +1994,7 @@ class FSROOverlay(FSOverlayBase["FSROOverlay"]):
         unlink_if_exists: bool = True,
         use_fs_path_mode: bool = False,
         mode: int = 0o0644,
-        mtime: Optional[float] = None,
+        mtime: float | None = None,
     ) -> ContextManager["VirtualPath"]:
         self._error_ro_fs()
 
@@ -2101,8 +2099,8 @@ class FSControlPath(FSOverlayBase["FSControlPath"]):
 
     def chown(
         self,
-        owner: Optional[StaticFileSystemOwner],
-        group: Optional[StaticFileSystemGroup],
+        owner: StaticFileSystemOwner | None,
+        group: StaticFileSystemGroup | None,
     ) -> None:
         raise ValueError(
             "No need to chown paths in the control.tar: They are always root:root"
@@ -2119,7 +2117,7 @@ class FSControlPath(FSOverlayBase["FSControlPath"]):
         unlink_if_exists: bool = True,
         use_fs_path_mode: bool = False,
         mode: int = 0o0644,
-        mtime: Optional[float] = None,
+        mtime: float | None = None,
     ) -> ContextManager["VirtualPath"]:
         if "/" in name or name in {".", ".."}:
             raise ValueError(f'Invalid file name: "{name}"')
@@ -2186,21 +2184,63 @@ class FSControlRootDir(FSControlPath):
     def create_root_dir(cls, fs_path: str) -> "FSControlRootDir":
         return FSControlRootDir(".", fs_path, None)
 
+    def insert_file_from_fs_path(
+        self,
+        name: str,
+        fs_path: str,
+        *,
+        exist_ok: bool = True,
+        use_fs_path_mode: bool = False,
+        mode: int = 0o0644,
+        # Ignored, but accepted for compat with FSPath's variant of this function.
+        # - This is used by install_or_generate_conffiles.
+        reference_path: VirtualPath | None = None,  # noqa
+    ) -> "FSControlPath":
+        if "/" in name or name in {".", ".."}:
+            raise ValueError(f'Invalid file name: "{name}"')
+        if not self.is_dir:
+            raise TypeError(
+                f"Cannot create {self._orphan_safe_path()}/{name}:"
+                f" {self._orphan_safe_path()} is not a directory"
+            )
+        self._rw_check()
+        if name in self and not exist_ok:
+            raise ValueError(
+                f'The path "{self._orphan_safe_path()}" already contains a file called "{name}"'
+                f" and exist_ok was False"
+            )
 
-def as_path_def(pd: Union[str, PathDef]) -> PathDef:
+        target_path = os.path.join(self.fs_path, name)
+        if use_fs_path_mode:
+            shutil.copymode(
+                fs_path,
+                target_path,
+                follow_symlinks=True,
+            )
+        else:
+            shutil.copyfile(
+                fs_path,
+                target_path,
+                follow_symlinks=True,
+            )
+            os.chmod(target_path, mode)
+        return cast("FSControlPath", self[name])
+
+
+def as_path_def(pd: str | PathDef) -> PathDef:
     return PathDef(pd) if isinstance(pd, str) else pd
 
 
-def as_path_defs(paths: Iterable[Union[str, PathDef]]) -> Iterable[PathDef]:
+def as_path_defs(paths: Iterable[str | PathDef]) -> Iterable[PathDef]:
     yield from (as_path_def(p) for p in paths)
 
 
 def build_virtual_fs(
-    paths: Iterable[Union[str, PathDef]],
+    paths: Iterable[str | PathDef],
     read_write_fs: bool = False,
 ) -> "FSPath":
-    root_dir: Optional[FSRootDir] = None
-    directories: Dict[str, FSPath] = {}
+    root_dir: FSRootDir | None = None
+    directories: dict[str, FSPath] = {}
     non_directories = set()
 
     def _ensure_parent_dirs(p: str) -> None:
