@@ -7,22 +7,18 @@ from typing import (
     List,
     Dict,
     FrozenSet,
-    Callable,
     Union,
-    Iterator,
     Tuple,
     Set,
-    Sequence,
     Optional,
-    Iterable,
     TYPE_CHECKING,
     cast,
     Any,
-    Mapping,
 )
+from collections.abc import Callable, Iterator, Sequence, Iterable, Mapping
 
 from debputy.exceptions import DebputyRuntimeError
-from debputy.filesystem_scan import FSPath, FSRootDir
+from debputy.filesystem_scan import FSPath
 from debputy.manifest_conditions import (
     ConditionContext,
     ManifestCondition,
@@ -66,7 +62,7 @@ class PathAlreadyInstalledOrDiscardedError(InstallRuleError):
         return cast("str", self.args[0])
 
     @property
-    def into(self) -> FrozenSet[BinaryPackage]:
+    def into(self) -> frozenset[BinaryPackage]:
         return cast("FrozenSet[BinaryPackage]", self.args[1])
 
     @property
@@ -105,7 +101,7 @@ class NoMatchForInstallPatternError(InstallRuleError):
 @dataclasses.dataclass(slots=True, frozen=True)
 class SearchDir:
     search_dir: "VirtualPath"
-    applies_to: FrozenSet[BinaryPackage]
+    applies_to: frozenset[BinaryPackage]
 
 
 @dataclasses.dataclass(slots=True, frozen=True)
@@ -132,7 +128,7 @@ class InstallSearchDirContext:
 class InstallRuleContext:
     # TODO: Search dirs should be per-package
     search_dirs: Sequence[SearchDir]
-    binary_package_contexts: Dict[str, BinaryPackageInstallRuleContext] = (
+    binary_package_contexts: dict[str, BinaryPackageInstallRuleContext] = (
         dataclasses.field(default_factory=dict)
     )
 
@@ -151,7 +147,7 @@ class PathMatch:
     path: "VirtualPath"
     search_dir: "VirtualPath"
     is_exact_match: bool
-    into: FrozenSet[BinaryPackage]
+    into: frozenset[BinaryPackage]
 
 
 class DiscardState(IntEnum):
@@ -163,13 +159,13 @@ class DiscardState(IntEnum):
 
 def _determine_manpage_section(
     match_rule: PathMatch,
-    provided_section: Optional[int],
+    provided_section: int | None,
     definition_source: str,
-) -> Optional[str]:
+) -> str | None:
     section = str(provided_section) if provided_section is not None else None
     if section is None:
         detected_section = None
-        with open(match_rule.path.fs_path, "r") as fd:
+        with open(match_rule.path.fs_path) as fd:
             for line in fd:
                 if not line.startswith((".TH", ".Dt")):
                     continue
@@ -198,7 +194,7 @@ def _determine_manpage_section(
 
 def _determine_manpage_real_section(
     match_rule: PathMatch,
-    section: Optional[str],
+    section: str | None,
     definition_source: str,
 ) -> int:
     real_section = None
@@ -223,8 +219,8 @@ def _determine_manpage_real_section(
 
 def _determine_manpage_language(
     match_rule: PathMatch,
-    provided_language: Optional[str],
-) -> Optional[str]:
+    provided_language: str | None,
+) -> str | None:
     if provided_language is not None:
         if provided_language not in ("derive-from-basename", "derive-from-path"):
             return provided_language if provided_language != "C" else None
@@ -241,8 +237,8 @@ def _determine_manpage_language(
 
 
 def _dest_path_for_manpage(
-    provided_section: Optional[int],
-    provided_language: Optional[str],
+    provided_section: int | None,
+    provided_language: str | None,
     definition_source: str,
 ) -> Callable[["PathMatch"], str]:
     def _manpage_dest_path(match_rule: PathMatch) -> str:
@@ -271,15 +267,15 @@ def _dest_path_for_manpage(
 
 
 class SourcePathMatcher:
-    def __init__(self, auto_discard_rules: List["PluginProvidedDiscardRule"]) -> None:
-        self._already_matched: Dict[
+    def __init__(self, auto_discard_rules: list["PluginProvidedDiscardRule"]) -> None:
+        self._already_matched: dict[
             str,
-            Tuple[FrozenSet[BinaryPackage], str],
+            tuple[frozenset[BinaryPackage], str],
         ] = {}
-        self._exact_match_request: Set[Tuple[str, str]] = set()
-        self._discarded: Dict[str, DiscardState] = {}
+        self._exact_match_request: set[tuple[str, str]] = set()
+        self._discarded: dict[str, DiscardState] = {}
         self._auto_discard_rules = auto_discard_rules
-        self.used_auto_discard_rules: Dict[str, Set[str]] = collections.defaultdict(set)
+        self.used_auto_discard_rules: dict[str, set[str]] = collections.defaultdict(set)
 
     def is_reserved(self, path: "VirtualPath") -> bool:
         fs_path = path.fs_path
@@ -337,7 +333,7 @@ class SourcePathMatcher:
         match: PathMatch,
         *,
         is_exact_match: bool = False,
-    ) -> Tuple[FrozenSet[BinaryPackage], bool]:
+    ) -> tuple[frozenset[BinaryPackage], bool]:
         m = self._already_matched.get(match.path.fs_path)
         if m:
             return m[0], False
@@ -360,7 +356,7 @@ class SourcePathMatcher:
     def reserve(
         self,
         path: "VirtualPath",
-        reserved_by: FrozenSet[BinaryPackage],
+        reserved_by: frozenset[BinaryPackage],
         definition_source: str,
         *,
         is_exact_match: bool = False,
@@ -398,10 +394,10 @@ class SourcePathMatcher:
         match_rule: MatchRule,
         search_dirs: Sequence[SearchDir],
         dir_only_match: bool,
-        match_filter: Optional[Callable[["VirtualPath"], bool]],
-        reserved_by: FrozenSet[BinaryPackage],
+        match_filter: Callable[["VirtualPath"], bool] | None,
+        reserved_by: frozenset[BinaryPackage],
         definition_source: str,
-    ) -> Tuple[List[PathMatch], Tuple[int, ...]]:
+    ) -> tuple[list[PathMatch], tuple[int, ...]]:
         matched = []
         already_installed_paths = 0
         already_excluded_paths = 0
@@ -470,8 +466,8 @@ def _resolve_path(
     match_rule: MatchRule,
     search_dirs: Iterable["SearchDir"],
     dir_only_match: bool,
-    match_filter: Optional[Callable[["VirtualPath"], bool]],
-    into: FrozenSet[BinaryPackage],
+    match_filter: Callable[["VirtualPath"], bool] | None,
+    into: frozenset[BinaryPackage],
 ) -> Iterator[PathMatch]:
     missing_matches = set(into)
     for sdir in search_dirs:
@@ -508,9 +504,9 @@ def _resolve_path(
 
 def _resolve_dest_paths(
     match: PathMatch,
-    dest_paths: Sequence[Tuple[str, bool]],
+    dest_paths: Sequence[tuple[str, bool]],
     install_context: "InstallRuleContext",
-) -> Sequence[Tuple[str, "FSPath"]]:
+) -> Sequence[tuple[str, "FSPath"]]:
     dest_and_roots = []
     for dest_path, dest_path_is_format in dest_paths:
         if dest_path_is_format:
@@ -543,11 +539,11 @@ def _resolve_dest_paths(
 
 
 def _resolve_matches(
-    matches: List[PathMatch],
-    dest_paths: Union[Sequence[Tuple[str, bool]], Callable[[PathMatch], str]],
+    matches: list[PathMatch],
+    dest_paths: Sequence[tuple[str, bool]] | Callable[[PathMatch], str],
     install_context: "InstallRuleContext",
-) -> Iterator[Tuple[PathMatch, Sequence[Tuple[str, "FSPath"]]]]:
-    dest_and_roots: Sequence[Tuple[str, "FSPath"]]
+) -> Iterator[tuple[PathMatch, Sequence[tuple[str, "FSPath"]]]]:
+    dest_and_roots: Sequence[tuple[str, "FSPath"]]
     if callable(dest_paths):
         compute_dest_path = dest_paths
         for match in matches:
@@ -581,10 +577,10 @@ class InstallRule(DebputyDispatchableType):
 
     def __init__(
         self,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
         definition_source: str,
         *,
-        match_filter: Optional[Callable[["VirtualPath"], bool]] = None,
+        match_filter: Callable[["VirtualPath"], bool] | None = None,
     ) -> None:
         super().__init__()
         self._condition = condition
@@ -592,7 +588,7 @@ class InstallRule(DebputyDispatchableType):
         self._match_filter = match_filter
 
     def _check_single_match(
-        self, source: FileSystemMatchRule, matches: List[PathMatch]
+        self, source: FileSystemMatchRule, matches: list[PathMatch]
     ) -> None:
         seen_pkgs = set()
         problem_pkgs = frozenset()
@@ -615,8 +611,8 @@ class InstallRule(DebputyDispatchableType):
         fs_match_rule: FileSystemMatchRule,
         condition_context: ConditionContext,
         search_dirs: Sequence[SearchDir],
-        into: FrozenSet[BinaryPackage],
-    ) -> List[PathMatch]:
+        into: frozenset[BinaryPackage],
+    ) -> list[PathMatch]:
         (matched, exclude_counts) = path_matcher.find_and_reserve_all_matches(
             fs_match_rule.match_rule,
             search_dirs,
@@ -691,10 +687,10 @@ class InstallRule(DebputyDispatchableType):
     def _install_matches(
         self,
         path_matcher: SourcePathMatcher,
-        matches: List[PathMatch],
-        dest_paths: Union[Sequence[Tuple[str, bool]], Callable[[PathMatch], str]],
+        matches: list[PathMatch],
+        dest_paths: Sequence[tuple[str, bool]] | Callable[[PathMatch], str],
         install_context: "InstallRuleContext",
-        into: FrozenSet[BinaryPackage],
+        into: frozenset[BinaryPackage],
         condition_context: ConditionContext,
     ) -> None:
         if (
@@ -705,22 +701,13 @@ class InstallRule(DebputyDispatchableType):
             # for this particular case. Though, we do go through the match and
             # ensure all paths are reserved correctly such that the behavior
             # between `binary-arch` vs `binary-indep` vs `binary` is consistent
-            root = FSRootDir()
             for match in matches:
-                path = match.path
-                path_matcher.reserve(
-                    path,
+                self._reserve_recursively(
+                    path_matcher,
+                    match,
                     into,
-                    self._definition_source,
-                    is_exact_match=False,
                 )
-                if path.is_dir:
-                    self._install_dir_recursively(
-                        path_matcher,
-                        [root],
-                        match,
-                        into,
-                    )
+
             return
 
         if not matches:
@@ -781,12 +768,39 @@ class InstallRule(DebputyDispatchableType):
                     into,
                 )
 
+    def _reserve_recursively(
+        self,
+        path_matcher: SourcePathMatcher,
+        match: PathMatch,
+        into: frozenset[BinaryPackage],
+    ) -> None:
+        direct_matched_path = match.path
+        path_matcher.reserve(
+            direct_matched_path,
+            into,
+            self._definition_source,
+            is_exact_match=match.is_exact_match,
+        )
+        if not direct_matched_path.is_dir:
+            return
+        stack = list(direct_matched_path.iterdir)
+        while stack:
+            current_path = stack.pop()
+            path_matcher.reserve(
+                current_path,
+                into,
+                self._definition_source,
+                is_exact_match=False,
+            )
+            if current_path.is_dir:
+                stack.extend(current_path.iterdir)
+
     def _install_dir_recursively(
         self,
         path_matcher: SourcePathMatcher,
         parent_dirs: Sequence[FSPath],
         match: PathMatch,
-        into: FrozenSet[BinaryPackage],
+        into: frozenset[BinaryPackage],
     ) -> None:
         stack = [
             (parent_dirs, e)
@@ -846,9 +860,9 @@ class InstallRule(DebputyDispatchableType):
         cls,
         source: FileSystemMatchRule,
         dest_path: str,
-        into: FrozenSet[BinaryPackage],
+        into: frozenset[BinaryPackage],
         definition_source: str,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
     ) -> "InstallRule":
         return GenericInstallationRule(
             [source],
@@ -863,10 +877,10 @@ class InstallRule(DebputyDispatchableType):
     def install_dest(
         cls,
         sources: Sequence[FileSystemMatchRule],
-        dest_dir: Optional[str],
-        into: FrozenSet[BinaryPackage],
+        dest_dir: str | None,
+        into: frozenset[BinaryPackage],
         definition_source: str,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
     ) -> "InstallRule":
         if dest_dir is None:
             dest_dir = "{dirname}/{basename}"
@@ -885,9 +899,9 @@ class InstallRule(DebputyDispatchableType):
         cls,
         source: FileSystemMatchRule,
         dest_paths: Sequence[str],
-        into: FrozenSet[BinaryPackage],
+        into: frozenset[BinaryPackage],
         definition_source: str,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
     ) -> "InstallRule":
         if len(dest_paths) < 2:
             raise ValueError(
@@ -908,9 +922,9 @@ class InstallRule(DebputyDispatchableType):
         cls,
         sources: Sequence[FileSystemMatchRule],
         dest_dirs: Sequence[str],
-        into: FrozenSet[BinaryPackage],
+        into: frozenset[BinaryPackage],
         definition_source: str,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
     ) -> "InstallRule":
         if len(dest_dirs) < 2:
             raise ValueError(
@@ -929,10 +943,10 @@ class InstallRule(DebputyDispatchableType):
     def install_doc(
         cls,
         sources: Sequence[FileSystemMatchRule],
-        dest_dir: Optional[str],
-        into: FrozenSet[BinaryPackage],
+        dest_dir: str | None,
+        into: frozenset[BinaryPackage],
         definition_source: str,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
     ) -> "InstallRule":
         cond: ManifestCondition = _BUILD_DOCS_BDO
         if condition is not None:
@@ -955,9 +969,9 @@ class InstallRule(DebputyDispatchableType):
         cls,
         source: FileSystemMatchRule,
         dest_path: str,
-        into: FrozenSet[BinaryPackage],
+        into: frozenset[BinaryPackage],
         definition_source: str,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
     ) -> "InstallRule":
         cond: ManifestCondition = _BUILD_DOCS_BDO
         if condition is not None:
@@ -976,9 +990,9 @@ class InstallRule(DebputyDispatchableType):
     def install_examples(
         cls,
         sources: Sequence[FileSystemMatchRule],
-        into: FrozenSet[BinaryPackage],
+        into: frozenset[BinaryPackage],
         definition_source: str,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
     ) -> "InstallRule":
         cond: ManifestCondition = _BUILD_DOCS_BDO
         if condition is not None:
@@ -995,11 +1009,11 @@ class InstallRule(DebputyDispatchableType):
     def install_man(
         cls,
         sources: Sequence[FileSystemMatchRule],
-        into: FrozenSet[BinaryPackage],
-        section: Optional[int],
-        language: Optional[str],
+        into: frozenset[BinaryPackage],
+        section: int | None,
+        language: str | None,
         definition_source: str,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
     ) -> "InstallRule":
         cond: ManifestCondition = _BUILD_DOCS_BDO
         if condition is not None:
@@ -1023,9 +1037,9 @@ class InstallRule(DebputyDispatchableType):
         cls,
         paths: Sequence[FileSystemMatchRule],
         definition_source: str,
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
         *,
-        limit_to: Optional[Sequence[FileSystemExactMatchRule]] = None,
+        limit_to: Sequence[FileSystemExactMatchRule] | None = None,
     ) -> "InstallRule":
         return DiscardRule(
             paths,
@@ -1092,13 +1106,13 @@ class GenericInstallationRule(InstallRule):
     def __init__(
         self,
         sources: Sequence[FileSystemMatchRule],
-        dest_paths: Union[Sequence[Tuple[str, bool]], Callable[[PathMatch], str]],
-        into: FrozenSet[BinaryPackage],
-        condition: Optional[ManifestCondition],
+        dest_paths: Sequence[tuple[str, bool]] | Callable[[PathMatch], str],
+        into: frozenset[BinaryPackage],
+        condition: ManifestCondition | None,
         definition_source: str,
         *,
         require_single_match: bool = False,
-        match_filter: Optional[Callable[["VirtualPath"], bool]] = None,
+        match_filter: Callable[["VirtualPath"], bool] | None = None,
     ) -> None:
         super().__init__(
             condition,
@@ -1144,7 +1158,7 @@ class DiscardRule(InstallRule):
     def __init__(
         self,
         fs_match_rules: Sequence[FileSystemMatchRule],
-        condition: Optional[ManifestCondition],
+        condition: ManifestCondition | None,
         limit_to: Sequence[FileSystemExactMatchRule],
         definition_source: str,
     ) -> None:
