@@ -1,20 +1,15 @@
 import dataclasses
+import difflib
 import os
 import stat
-import subprocess
 import sys
 import textwrap
 from contextlib import suppress
 from typing import (
-    Optional,
-    List,
-    Union,
     NoReturn,
-    Mapping,
-    Sequence,
     TYPE_CHECKING,
-    Callable,
 )
+from collections.abc import Mapping, Sequence, Callable
 
 from debputy.commands.debputy_cmd.context import CommandContext
 from debputy.commands.debputy_cmd.output import _output_styling, IOBasedOutputStyling
@@ -29,31 +24,7 @@ from debputy.linting.lint_util import (
 )
 from debputy.lsp.config.debputy_config import DebputyConfig, load_debputy_config
 from debputy.lsp.diagnostics import DiagnosticData
-from debputy.lsp.languages.lsp_debian_changelog import _lint_debian_changelog
-from debputy.lsp.languages.lsp_debian_control import (
-    _lint_debian_control,
-    _reformat_debian_control,
-)
-from debputy.lsp.languages.lsp_debian_copyright import (
-    _lint_debian_copyright,
-    _reformat_debian_copyright,
-)
-from debputy.lsp.languages.lsp_debian_debputy_manifest import (
-    _lint_debian_debputy_manifest,
-)
-from debputy.lsp.languages.lsp_debian_patches_series import _lint_debian_patches_series
-from debputy.lsp.languages.lsp_debian_rules import _lint_debian_rules
-from debputy.lsp.languages.lsp_debian_tests_control import (
-    _lint_debian_tests_control,
-    _reformat_debian_tests_control,
-)
-from debputy.lsp.languages.lsp_debian_upstream_metadata import (
-    _lint_debian_upstream_metadata,
-)
-from debputy.lsp.languages.lsp_debian_watch import (
-    _lint_debian_watch,
-    _reformat_debian_watch,
-)
+from debputy.lsp.lsp_features import CLI_FORMAT_FILE_HANDLERS, CLI_DIAGNOSTIC_HANDLERS
 from debputy.lsp.maint_prefs import (
     MaintainerPreferenceTable,
     EffectiveFormattingPreference,
@@ -84,48 +55,31 @@ else:
     import debputy.lsprotocol.types as types
 
 
-LINTER_FORMATS = {
-    "debian/changelog": _lint_debian_changelog,
-    "debian/control": _lint_debian_control,
-    "debian/copyright": _lint_debian_copyright,
-    "debian/debputy.manifest": _lint_debian_debputy_manifest,
-    "debian/upstream/metadata": _lint_debian_upstream_metadata,
-    "debian/rules": _lint_debian_rules,
-    "debian/patches/series": _lint_debian_patches_series,
-    "debian/tests/control": _lint_debian_tests_control,
-    "debian/watch": _lint_debian_watch,
-}
-
-
-REFORMAT_FORMATS = {
-    "debian/control": _reformat_debian_control,
-    "debian/copyright": _reformat_debian_copyright,
-    "debian/tests/control": _reformat_debian_tests_control,
-    "debian/watch": _reformat_debian_watch,
-}
+LINTER_FORMATS = CLI_DIAGNOSTIC_HANDLERS
+REFORMAT_FORMATS = CLI_FORMAT_FILE_HANDLERS
 
 
 @dataclasses.dataclass(slots=True)
 class LintContext:
     plugin_feature_set: PluginProvidedFeatureSet
     maint_preference_table: MaintainerPreferenceTable
-    source_root: Optional[VirtualPath]
-    debian_dir: Optional[VirtualPath]
+    source_root: VirtualPath | None
+    debian_dir: VirtualPath | None
     debputy_config: DebputyConfig
-    parsed_deb822_file_content: Optional[Deb822FileElement] = None
-    source_package: Optional[SourcePackage] = None
-    binary_packages: Optional[Mapping[str, BinaryPackage]] = None
-    effective_preference: Optional[EffectiveFormattingPreference] = None
-    style_tool: Optional[str] = None
-    unsupported_preference_reason: Optional[str] = None
-    salsa_ci: Optional[CommentedMap] = None
+    parsed_deb822_file_content: Deb822FileElement | None = None
+    source_package: SourcePackage | None = None
+    binary_packages: Mapping[str, BinaryPackage] | None = None
+    effective_preference: EffectiveFormattingPreference | None = None
+    style_tool: str | None = None
+    unsupported_preference_reason: str | None = None
+    salsa_ci: CommentedMap | None = None
 
     def state_for(
         self,
         path: str,
         content: str,
-        lines: List[str],
-        lint_implementation: Optional[AsyncLinterImpl],
+        lines: list[str],
+        lint_implementation: AsyncLinterImpl | None,
     ) -> LintStateImpl:
         return LintStateImpl(
             self.plugin_feature_set,
@@ -166,8 +120,8 @@ def gather_lint_info(context: CommandContext) -> LintContext:
         lint_context.parsed_deb822_file_content = deb822_file
         lint_context.source_package = source_package
         lint_context.binary_packages = binary_packages
-    salsa_ci_map: Optional[CommentedMap] = None
-    for ci_file in ("debian/salsa-ci.yml", ".gitlab-ci.yml"):
+    salsa_ci_map: CommentedMap | None = None
+    for ci_file in ("debian/salsa-ci.yml", "debian/gitlab-ci.yml", ".gitlab-ci.yml"):
         try:
             with open(ci_file) as fd:
                 salsa_ci_map = MANIFEST_YAML.load(fd)
@@ -262,7 +216,7 @@ async def perform_linting(context: CommandContext) -> None:
 def perform_reformat(
     context: CommandContext,
     *,
-    named_style: Optional[str] = None,
+    named_style: str | None = None,
 ) -> None:
     parsed_args = context.parsed_args
     fo = _output_styling(context.parsed_args, sys.stdout)
@@ -319,9 +273,9 @@ def perform_reformat(
                  * If you use the Debian Salsa CI pipeline, then you can set SALSA_CI_DISABLE_WRAP_AND_SORT (`no`)
                    or SALSA_CI_ENABLE_WRAP_AND_SORT (`yes`) to the relevant value and `debputy` will pick up the
                    configuration from there.
-                   - Note: The option must be in `.gitlab-ci.yml` or `debian/salsa-ci.yml` to work. The Salsa CI
-                     pipeline will use `wrap-and-sort` while `debputy` uses its own emulation of `wrap-and-sort`
-                     (`debputy` also needs to apply the style via `debputy lsp server`).
+                   - Note: The option must be in `.gitlab-ci.yml`, `debian/gitlab-ci.yml` or `debian/salsa-ci.yml`
+                     to work. The Salsa CI pipeline will use `wrap-and-sort` while `debputy` uses its own emulation
+                     of `wrap-and-sort` (`debputy` also needs to apply the style via `debputy lsp server`).
 
                  * The `debputy` code also comes with a built-in style database. This may be interesting for
                    packaging teams, so set a default team style that applies to all packages maintained by
@@ -412,9 +366,9 @@ def perform_reformat_of_file(
     formatter: FormatterImpl,
     auto_fix: bool,
     *,
-    custom_modifier: Optional[Callable[[str], str]] = None,
+    custom_modifier: Callable[[str], str] | None = None,
 ) -> bool:
-    with open(filename, "rt", encoding="utf-8") as fd:
+    with open(filename, encoding="utf-8") as fd:
         text = fd.read()
 
     lines = text.splitlines(keepends=True)
@@ -442,14 +396,19 @@ def perform_reformat_of_file(
     if custom_modifier:
         replacement = custom_modifier(replacement)
 
-    output_filename = f"{filename}.tmp"
-    with open(output_filename, "wt", encoding="utf-8") as fd:
-        fd.write(replacement)
+    unified_diff = difflib.unified_diff(
+        text.splitlines(keepends=True),
+        replacement.splitlines(keepends=True),
+        fromfile=filename,
+        tofile=filename,
+    )
+    for line in unified_diff:
+        print(line, end="")
 
-    r = subprocess.run(["diff", "-u", filename, output_filename]).returncode
-    if r != 0 and r != 1:
-        _warn(f"diff -u {filename} {output_filename} failed!?")
     if auto_fix:
+        output_filename = f"{filename}.tmp"
+        with open(output_filename, "w", encoding="utf-8") as fd:
+            fd.write(replacement)
         orig_mode = stat.S_IMODE(os.stat(filename).st_mode)
         os.chmod(output_filename, orig_mode)
         os.rename(output_filename, filename)
@@ -460,8 +419,6 @@ def perform_reformat_of_file(
                 style="bold",
             )
         )
-    else:
-        os.unlink(output_filename)
 
     return True
 
@@ -486,7 +443,7 @@ async def perform_linting_of_file(
     handler = LINTER_FORMATS.get(file_format)
     if handler is None:
         return
-    with open(filename, "rt", encoding="utf-8") as fd:
+    with open(filename, encoding="utf-8") as fd:
         text = fd.read()
 
     if auto_fixing_enabled:
@@ -549,7 +506,7 @@ _INITIAL_FIX_RANGE = types.Range(
 
 
 def _is_non_interactive_auto_fix_allowed(diagnostic: types.Diagnostic) -> bool:
-    diag_data: Optional[DiagnosticData] = diagnostic.data
+    diag_data: DiagnosticData | None = diagnostic.data
     return diag_data is None or diag_data.get("enable_non_interactive_auto_fix", True)
 
 
@@ -561,7 +518,7 @@ async def _auto_fix_run(
     lint_report: LintReport,
 ) -> None:
     another_round = True
-    unfixed_diagnostics: List[types.Diagnostic] = []
+    unfixed_diagnostics: list[types.Diagnostic] = []
     remaining_rounds = 10
     fixed_count = 0
     too_many_rounds = False
@@ -656,7 +613,7 @@ async def _auto_fix_run(
 
     if fixed_count:
         output_filename = f"{filename}.tmp"
-        with open(output_filename, "wt", encoding="utf-8") as fd:
+        with open(output_filename, "w", encoding="utf-8") as fd:
             fd.write(text)
         orig_mode = stat.S_IMODE(os.stat(filename).st_mode)
         os.chmod(output_filename, orig_mode)
@@ -756,7 +713,7 @@ async def _diagnostics_run(
 
 
 def has_at_least_lsp_quickfix(
-    actions: Optional[List[Union[types.Command, types.CodeAction]]],
+    actions: list[types.Command | types.CodeAction] | None,
 ) -> bool:
     if actions is None:
         return False
@@ -774,8 +731,8 @@ def has_at_least_lsp_quickfix(
 
 def resolve_auto_fixer(
     document_ref: str,
-    actions: Optional[List[Union[types.Command, types.CodeAction]]],
-) -> Sequence[Union[types.TextEdit, types.AnnotatedTextEdit]]:
+    actions: list[types.Command | types.CodeAction] | None,
+) -> Sequence[types.TextEdit | types.AnnotatedTextEdit]:
     if actions is None or len(actions) != 1:
         return tuple()
     action = actions[0]
