@@ -11,20 +11,22 @@ from functools import lru_cache
 from typing import (
     List,
     Optional,
-    Callable,
     TYPE_CHECKING,
-    Mapping,
-    Sequence,
     cast,
     FrozenSet,
     Self,
     Set,
     Dict,
+    Tuple,
+    TypeVar,
+)
+from collections.abc import (
+    Callable,
+    Mapping,
+    Sequence,
     Iterable,
     Awaitable,
     AsyncIterable,
-    Tuple,
-    TypeVar,
 )
 
 
@@ -83,7 +85,7 @@ class AbortTaskError(DebputyRuntimeError):
 
 
 # If you add a new one to this set, remember to mention it in the docs of `LintState.emit_diagnostic`
-DIAG_SOURCE_WITHOUT_SECTIONS: FrozenSet[str] = frozenset(
+DIAG_SOURCE_WITHOUT_SECTIONS: frozenset[str] = frozenset(
     {
         "debputy",
         "dpkg",
@@ -91,7 +93,7 @@ DIAG_SOURCE_WITHOUT_SECTIONS: FrozenSet[str] = frozenset(
 )
 
 # If you add a new one to this set, remember to mention it in the docs of `LintState.emit_diagnostic`
-DIAG_SOURCE_WITH_SECTIONS: FrozenSet[str] = frozenset(
+DIAG_SOURCE_WITH_SECTIONS: frozenset[str] = frozenset(
     {
         "Policy",
         "DevRef",
@@ -117,7 +119,7 @@ def with_range_in_continuous_parts(
     iterable: Iterable["L"],
     *,
     start_relative_to: "TEPosition" = START_POSITION,
-) -> Iterable[Tuple["TERange", "L"]]:
+) -> Iterable[tuple["TERange", "L"]]:
     current_pos = start_relative_to
     for part in iterable:
         part_range = part.size().relative_to(current_pos)
@@ -143,7 +145,7 @@ def _check_diagnostic_source(source: str) -> None:
 
 @dataclasses.dataclass(slots=True)
 class DebputyMetadata:
-    debputy_integration_mode: Optional[DebputyIntegrationMode]
+    debputy_integration_mode: DebputyIntegrationMode | None
 
     @classmethod
     def from_data(
@@ -211,11 +213,11 @@ class LintState:
         raise NotImplementedError
 
     @property
-    def doc_version(self) -> Optional[int]:
+    def doc_version(self) -> int | None:
         raise NotImplementedError
 
     @property
-    def source_root(self) -> Optional[VirtualPathBase]:
+    def source_root(self) -> VirtualPathBase | None:
         """The path to the unpacked source root directory if available
 
         This is the directory that would contain the `debian/` directory. Note, if you need the
@@ -225,7 +227,7 @@ class LintState:
         raise NotImplementedError
 
     @property
-    def debian_dir(self) -> Optional[VirtualPathBase]:
+    def debian_dir(self) -> VirtualPathBase | None:
         """The path to the `debian/` directory if available"""
         raise NotImplementedError
 
@@ -245,7 +247,7 @@ class LintState:
         raise NotImplementedError
 
     @property
-    def lines(self) -> List[str]:
+    def lines(self) -> list[str]:
         # FIXME: Replace with `Sequence[str]` if possible
         """The contents of the file being checked as a list of lines
 
@@ -258,7 +260,7 @@ class LintState:
         raise NotImplementedError
 
     @property
-    def parsed_deb822_file_content(self) -> Optional[Deb822FileElement]:
+    def parsed_deb822_file_content(self) -> Deb822FileElement | None:
         """The contents of the file being checked as a parsed deb822 file
 
         This can sometimes use a cached version of the parsed file and is therefore preferable to
@@ -269,7 +271,7 @@ class LintState:
         raise NotImplementedError
 
     @property
-    def source_package(self) -> Optional[SourcePackage]:
+    def source_package(self) -> SourcePackage | None:
         """The source package (source stanza of `debian/control`).
 
         Will be `None` if the `debian/control` file cannot be parsed as a deb822 file, or if the
@@ -278,7 +280,7 @@ class LintState:
         raise NotImplementedError
 
     @property
-    def binary_packages(self) -> Optional[Mapping[str, BinaryPackage]]:
+    def binary_packages(self) -> Mapping[str, BinaryPackage] | None:
         """The binary packages (the Package stanzas of `debian/control`).
 
         Will be `None` if the `debian/control` file cannot be parsed, or if no Package stanzas are
@@ -288,7 +290,7 @@ class LintState:
 
     @property
     def maint_preference_table(self) -> "MaintainerPreferenceTable":
-        # TODO: Remove (unused)
+        # TODO: Visible only for tests.
         raise NotImplementedError
 
     @property
@@ -351,7 +353,7 @@ class LintState:
         text_range: "TERange",
         message: str,
         *,
-        doc_uri: Optional[str] = None,
+        doc_uri: str | None = None,
     ) -> RelatedDiagnosticInformation:
         """Provide a related context for the diagnostic
 
@@ -379,10 +381,10 @@ class LintState:
         severity: LintSeverity,
         authority_reference: str,
         *,
-        quickfixes: Optional[List[dict]] = None,
-        tags: Optional[List[types.DiagnosticTag]] = None,
-        related_information: Optional[List[RelatedDiagnosticInformation]] = None,
-        diagnostic_applies_to_another_file: Optional[str] = None,
+        quickfixes: list[dict] | None = None,
+        tags: list[types.DiagnosticTag] | None = None,
+        related_information: list[RelatedDiagnosticInformation] | None = None,
+        diagnostic_applies_to_another_file: str | None = None,
         enable_non_interactive_auto_fix: bool = True,
     ) -> None:
         """Emit a diagnostic for an issue detected in the current file.
@@ -437,6 +439,11 @@ class LintState:
             te_range_to_lsp(text_range),
         )
 
+        if related_information and any(
+            i.doc_uri != self.doc_uri for i in related_information
+        ):
+            raise NotImplementedError("Ranges from another document will be wrong")
+
         related_lsp_format = (
             [i.to_lsp(self) for i in related_information]
             if related_information
@@ -466,19 +473,19 @@ CLI_WORKSPACE_TEXT_EDIT_SUPPORT = WorkspaceTextEditSupport(
 class LintStateImpl(LintState):
     plugin_feature_set: PluginProvidedFeatureSet = dataclasses.field(repr=False)
     maint_preference_table: "MaintainerPreferenceTable" = dataclasses.field(repr=False)
-    source_root: Optional[VirtualPathBase]
-    debian_dir: Optional[VirtualPathBase]
+    source_root: VirtualPathBase | None
+    debian_dir: VirtualPathBase | None
     path: str
     content: str
-    lines: List[str]
+    lines: list[str]
     debputy_config: DebputyConfig
-    source_package: Optional[SourcePackage] = None
-    binary_packages: Optional[Mapping[str, BinaryPackage]] = None
+    source_package: SourcePackage | None = None
+    binary_packages: Mapping[str, BinaryPackage] | None = None
     effective_preference: Optional["EffectiveFormattingPreference"] = None
     lint_implementation: Optional["AsyncLinterImpl"] = None
-    _parsed_cache: Optional[Deb822FileElement] = None
-    _dh_sequencer_cache: Optional[DhSequencerData] = None
-    _diagnostics: Optional[List[types.Diagnostic]] = None
+    _parsed_cache: Deb822FileElement | None = None
+    _dh_sequencer_cache: DhSequencerData | None = None
+    _diagnostics: list[types.Diagnostic] | None = None
 
     @property
     def doc_uri(self) -> str:
@@ -487,7 +494,7 @@ class LintStateImpl(LintState):
         return f"file://{abs_path}"
 
     @property
-    def doc_version(self) -> Optional[int]:
+    def doc_version(self) -> int | None:
         return None
 
     @property
@@ -495,7 +502,7 @@ class LintStateImpl(LintState):
         return LINTER_POSITION_CODEC
 
     @property
-    def parsed_deb822_file_content(self) -> Optional[Deb822FileElement]:
+    def parsed_deb822_file_content(self) -> Deb822FileElement | None:
         cache = self._parsed_cache
         if cache is None:
             cache = parse_deb822_file(
@@ -511,7 +518,7 @@ class LintStateImpl(LintState):
         dh_sequencer_cache = self._dh_sequencer_cache
         if dh_sequencer_cache is None:
             debian_dir = self.debian_dir
-            dh_sequences: Set[str] = set()
+            dh_sequences: set[str] = set()
             saw_dh = False
             src_pkg = self.source_package
             drules = debian_dir.get("rules") if debian_dir is not None else None
@@ -535,7 +542,7 @@ class LintStateImpl(LintState):
     def workspace_text_edit_support(self) -> WorkspaceTextEditSupport:
         return CLI_WORKSPACE_TEXT_EDIT_SUPPORT
 
-    async def gather_diagnostics(self) -> List[types.Diagnostic]:
+    async def gather_diagnostics(self) -> list[types.Diagnostic]:
         if self._diagnostics is not None:
             raise RuntimeError(
                 "run_diagnostics cannot be run while it is already running"
@@ -575,26 +582,26 @@ class LintDiagnosticResultState(IntEnum):
 class LintDiagnosticResult:
     diagnostic: types.Diagnostic
     result_state: LintDiagnosticResultState
-    invalid_marker: Optional[RuntimeError]
+    invalid_marker: RuntimeError | None
     is_file_level_diagnostic: bool
     has_broken_range: bool
     missing_severity: bool
     discovered_in: str
-    report_for_related_file: Optional[str]
+    report_for_related_file: str | None
 
 
 class LintReport:
 
     def __init__(self) -> None:
         self.diagnostics_count: Counter[types.DiagnosticSeverity] = Counter()
-        self.diagnostics_by_file: Mapping[str, List[LintDiagnosticResult]] = (
+        self.diagnostics_by_file: Mapping[str, list[LintDiagnosticResult]] = (
             defaultdict(list)
         )
         self.number_of_invalid_diagnostics: int = 0
         self.number_of_broken_diagnostics: int = 0
-        self.lint_state: Optional[LintState] = None
+        self.lint_state: LintState | None = None
         self.start_timestamp = datetime.datetime.now()
-        self.durations: Dict[str, float] = collections.defaultdict(lambda: 0.0)
+        self.durations: dict[str, float] = collections.defaultdict(float)
         self._timer = time.perf_counter()
 
     @contextlib.contextmanager
@@ -622,7 +629,7 @@ class LintReport:
         diagnostic: types.Diagnostic,
         *,
         result_state: LintDiagnosticResultState = LintDiagnosticResultState.REPORTED,
-        in_file: Optional[str] = None,
+        in_file: str | None = None,
     ) -> None:
         lint_state = self.lint_state
         assert lint_state is not None
@@ -631,7 +638,7 @@ class LintReport:
         discovered_in_file = in_file
         severity = diagnostic.severity
         missing_severity = False
-        error_marker: Optional[RuntimeError] = None
+        error_marker: RuntimeError | None = None
         if severity is None:
             self.number_of_invalid_diagnostics += 1
             severity = types.DiagnosticSeverity.Warning
@@ -739,7 +746,7 @@ _TERM_SEVERITY2TAG = {
 
 
 def debputy_severity(diagnostic: types.Diagnostic) -> LintSeverity:
-    lint_tag: Optional[LintSeverity] = None
+    lint_tag: LintSeverity | None = None
     if isinstance(diagnostic.data, dict):
         lint_tag = cast("LintSeverity", diagnostic.data.get("lint_severity"))
 
@@ -773,7 +780,7 @@ class TermLintReport(LintReport):
         assert severity is not None
         if diagnostic_result.result_state != LintDiagnosticResultState.FIXED:
             tag_unresolved = _TERM_SEVERITY2TAG[severity]
-            lint_tag: Optional[LintSeverity] = debputy_severity(diagnostic)
+            lint_tag: LintSeverity | None = debputy_severity(diagnostic)
             tag = tag_unresolved(fo, lint_tag)
         else:
             tag = fo.colored(
@@ -845,7 +852,7 @@ class TermLintReport(LintReport):
     def _print_range_context(
         self,
         print_range: types.Range,
-        lines: List[str],
+        lines: list[str],
         line_no_format_width: int,
     ) -> None:
         lines_to_print = _lines_to_print(print_range)
@@ -863,7 +870,7 @@ class LinterPositionCodec:
 
     def position_from_client_units(
         self,
-        lines: List[str],
+        lines: list[str],
         position: types.Position,
     ) -> types.Position:
 
@@ -875,18 +882,18 @@ class LinterPositionCodec:
 
     def position_to_client_units(
         self,
-        _lines: List[str],
+        _lines: list[str],
         position: types.Position,
     ) -> types.Position:
         return position
 
     def range_from_client_units(
-        self, _lines: List[str], range: types.Range
+        self, _lines: list[str], range: types.Range
     ) -> types.Range:
         return range
 
     def range_to_client_units(
-        self, _lines: List[str], range: types.Range
+        self, _lines: list[str], range: types.Range
     ) -> types.Range:
         return range
 
@@ -926,7 +933,7 @@ def _highlight_range(
 
 
 def _is_file_level_diagnostic(
-    lines: List[str],
+    lines: list[str],
     start_line: int,
     start_position: int,
     end_line: int,
