@@ -6,15 +6,13 @@ from functools import lru_cache
 from itertools import chain
 from typing import (
     Union,
-    Sequence,
     Tuple,
     Optional,
-    Mapping,
     List,
-    Iterable,
     Self,
     TYPE_CHECKING,
 )
+from collections.abc import Sequence, Mapping, Iterable
 
 import debputy.lsp.data.deb822_data as deb822_ref_data_dir
 from debputy.analysis.analysis_util import flatten_ppfs
@@ -137,7 +135,7 @@ _DISPATCH_RULE = LanguageDispatchRule.new_rule(
 class SubstvarMetadata:
     name: str
     defined_by: str
-    dh_sequence: Optional[str]
+    dh_sequence: str | None
     doc_uris: Sequence[str]
     synopsis: str
     description: str
@@ -170,7 +168,7 @@ class SubstvarMetadata:
         )
 
 
-def relationship_substvar_for_field(substvar: str) -> Optional[str]:
+def relationship_substvar_for_field(substvar: str) -> str | None:
     relationship_fields = all_package_relationship_fields()
     try:
         col_idx = substvar.rindex(":")
@@ -180,7 +178,7 @@ def relationship_substvar_for_field(substvar: str) -> Optional[str]:
 
 
 def _as_substvars_metadata(
-    args: List[SubstvarMetadata],
+    args: list[SubstvarMetadata],
 ) -> Mapping[str, SubstvarMetadata]:
     r = {s.name: s for s in args}
     assert len(r) == len(args)
@@ -217,7 +215,7 @@ lsp_standard_handler(_DISPATCH_RULE, TEXT_DOCUMENT_CODE_ACTION)
 def _debian_control_hover(
     ls: "DebputyLanguageServer",
     params: HoverParams,
-) -> Optional[Hover]:
+) -> Hover | None:
     return deb822_hover(ls, params, _DCTRL_FILE_METADATA, custom_handler=_custom_hover)
 
 
@@ -226,7 +224,7 @@ def _custom_hover_description(
     _known_field: DctrlKnownField,
     line: str,
     _word_at_position: str,
-) -> Optional[Union[Hover, str]]:
+) -> Hover | str | None:
     if line[0].isspace():
         return None
     try:
@@ -400,7 +398,7 @@ def _disclaimer(is_empty: bool) -> str:
 
 def _render_package_by_name(
     name: str, known_field: DctrlKnownField, is_empty: bool
-) -> Optional[str]:
+) -> str | None:
     if _is_bd_field(known_field) and name.startswith("dh-sequence-") and len(name) > 12:
         sequence = name[12:]
         return (
@@ -455,7 +453,7 @@ def _custom_hover_relationship_field(
     known_field: DctrlKnownField,
     _line: str,
     word_at_position: str,
-) -> Optional[Union[Hover, str]]:
+) -> Hover | str | None:
     apt_cache = ls.apt_cache
     state = apt_cache.state
     is_empty = False
@@ -524,13 +522,13 @@ _CUSTOM_FIELD_HOVER["Description"] = _custom_hover_description
 def _custom_hover(
     ls: "DebputyLanguageServer",
     server_position: Position,
-    _current_field: Optional[str],
+    _current_field: str | None,
     word_at_position: str,
-    known_field: Optional[DctrlKnownField],
+    known_field: DctrlKnownField | None,
     in_value: bool,
     _doc: "TextDocument",
-    lines: List[str],
-) -> Optional[Union[Hover, str]]:
+    lines: list[str],
+) -> Hover | str | None:
     if not in_value:
         return None
 
@@ -587,7 +585,7 @@ def _custom_hover(
 def _debian_control_completions(
     ls: "DebputyLanguageServer",
     params: CompletionParams,
-) -> Optional[Union[CompletionList, Sequence[CompletionItem]]]:
+) -> CompletionList | Sequence[CompletionItem] | None:
     return deb822_completer(ls, params, _DCTRL_FILE_METADATA)
 
 
@@ -595,7 +593,7 @@ def _debian_control_completions(
 def _debian_control_folding_ranges(
     ls: "DebputyLanguageServer",
     params: FoldingRangeParams,
-) -> Optional[Sequence[FoldingRange]]:
+) -> Sequence[FoldingRange] | None:
     return deb822_folding_ranges(ls, params, _DCTRL_FILE_METADATA)
 
 
@@ -603,7 +601,7 @@ def _debian_control_folding_ranges(
 async def _doc_inlay_hint(
     ls: "DebputyLanguageServer",
     params: types.InlayHintParams,
-) -> Optional[List[InlayHint]]:
+) -> list[InlayHint] | None:
     doc = ls.workspace.get_text_document(params.text_document.uri)
     lint_state = ls.lint_state(doc)
     deb822_file = lint_state.parsed_deb822_file_content
@@ -734,7 +732,7 @@ def _binary_package_checks(
     package_name = stanza.get("Package", "")
     source_section = source_stanza.get("Section")
     section_kvpair = stanza.get_kvpair_element(("Section", 0), use_get=True)
-    section: Optional[str] = None
+    section: str | None = None
     section_range: Optional["TERange"] = None
     if section_kvpair is not None:
         section, section_range = extract_first_value_and_position(
@@ -868,8 +866,8 @@ async def _lint_debian_control(lint_state: LintState) -> None:
 
 
 def _package_range_of_stanza(
-    binary_stanzas: List[Tuple[Deb822ParagraphElement, TEPosition]],
-) -> Iterable[Tuple[str, Optional[str], "TERange"]]:
+    binary_stanzas: list[tuple[Deb822ParagraphElement, TEPosition]],
+) -> Iterable[tuple[str, str | None, "TERange"]]:
     for stanza, stanza_position in binary_stanzas:
         kvpair = stanza.get_kvpair_element(("Package", 0), use_get=True)
         if kvpair is None:
@@ -934,6 +932,7 @@ def _packaging_files(
             all_dh_ppfs,
             _,
             _,
+            _,
         ) = resolve_debhelper_config_files(
             debian_dir,
             binary_packages,
@@ -955,7 +954,7 @@ def _packaging_files(
 
 def _detect_misspelled_packaging_files(
     lint_state: LintState,
-    binary_stanzas_w_pos: List[Tuple[Deb822ParagraphElement, TEPosition]],
+    binary_stanzas_w_pos: list[tuple[Deb822ParagraphElement, TEPosition]],
 ) -> None:
     stanza_ranges = {
         p: (a, r) for p, a, r in _package_range_of_stanza(binary_stanzas_w_pos)
@@ -1046,7 +1045,7 @@ def _detect_misspelled_packaging_files(
 def _debian_control_on_save_formatting(
     ls: "DebputyLanguageServer",
     params: WillSaveTextDocumentParams,
-) -> Optional[Sequence[TextEdit]]:
+) -> Sequence[TextEdit] | None:
     doc = ls.workspace.get_text_document(params.text_document.uri)
     lint_state = ls.lint_state(doc)
     return _reformat_debian_control(lint_state)
@@ -1055,7 +1054,7 @@ def _debian_control_on_save_formatting(
 @lsp_cli_reformat_document(_DISPATCH_RULE)
 def _reformat_debian_control(
     lint_state: LintState,
-) -> Optional[Sequence[TextEdit]]:
+) -> Sequence[TextEdit] | None:
     return deb822_format_file(lint_state, _DCTRL_FILE_METADATA)
 
 
@@ -1063,7 +1062,7 @@ def _reformat_debian_control(
 def _debian_control_format_file(
     ls: "DebputyLanguageServer",
     params: DocumentFormattingParams,
-) -> Optional[Sequence[TextEdit]]:
+) -> Sequence[TextEdit] | None:
     doc = ls.workspace.get_text_document(params.text_document.uri)
     lint_state = ls.lint_state(doc)
     return _reformat_debian_control(lint_state)
@@ -1073,7 +1072,7 @@ def _debian_control_format_file(
 async def _debian_control_semantic_tokens_full(
     ls: "DebputyLanguageServer",
     request: SemanticTokensParams,
-) -> Optional[SemanticTokens]:
+) -> SemanticTokens | None:
     return await deb822_semantic_tokens_full(
         ls,
         request,
