@@ -1,10 +1,10 @@
 import re
 from typing import (
     Union,
-    Sequence,
     Optional,
     List,
 )
+from collections.abc import Sequence
 
 from debputy.linting.lint_util import LintState, with_range_in_continuous_parts
 from debputy.lsp.debputy_ls import DebputyLanguageServer
@@ -22,6 +22,7 @@ from debputy.lsp.lsp_features import (
     lsp_format_document,
     SecondaryLanguage,
     LanguageDispatchRule,
+    lsp_cli_reformat_document,
 )
 from debputy.lsp.lsp_generic_deb822 import (
     deb822_completer,
@@ -86,7 +87,7 @@ lsp_standard_handler(_DISPATCH_RULE, TEXT_DOCUMENT_CODE_ACTION)
 def _debian_copyright_hover(
     ls: "DebputyLanguageServer",
     params: HoverParams,
-) -> Optional[Hover]:
+) -> Hover | None:
     return deb822_hover(ls, params, _DEP5_FILE_METADATA)
 
 
@@ -94,7 +95,7 @@ def _debian_copyright_hover(
 def _debian_copyright_completions(
     ls: "DebputyLanguageServer",
     params: CompletionParams,
-) -> Optional[Union[CompletionList, Sequence[CompletionItem]]]:
+) -> CompletionList | Sequence[CompletionItem] | None:
     return deb822_completer(ls, params, _DEP5_FILE_METADATA)
 
 
@@ -102,7 +103,7 @@ def _debian_copyright_completions(
 def _debian_copyright_folding_ranges(
     ls: "DebputyLanguageServer",
     params: FoldingRangeParams,
-) -> Optional[Sequence[FoldingRange]]:
+) -> Sequence[FoldingRange] | None:
     return deb822_folding_ranges(ls, params, _DEP5_FILE_METADATA)
 
 
@@ -158,15 +159,16 @@ async def _lint_debian_copyright(lint_state: LintState) -> None:
 def _debian_copyright_on_save_formatting(
     ls: "DebputyLanguageServer",
     params: WillSaveTextDocumentParams,
-) -> Optional[Sequence[TextEdit]]:
+) -> Sequence[TextEdit] | None:
     doc = ls.workspace.get_text_document(params.text_document.uri)
     lint_state = ls.lint_state(doc)
     return deb822_format_file(lint_state, _DEP5_FILE_METADATA)
 
 
+@lsp_cli_reformat_document(_DISPATCH_RULE)
 def _reformat_debian_copyright(
     lint_state: LintState,
-) -> Optional[Sequence[TextEdit]]:
+) -> Sequence[TextEdit] | None:
     return deb822_format_file(lint_state, _DEP5_FILE_METADATA)
 
 
@@ -174,7 +176,7 @@ def _reformat_debian_copyright(
 def _debian_copyright_format_document(
     ls: "DebputyLanguageServer",
     params: DocumentFormattingParams,
-) -> Optional[Sequence[TextEdit]]:
+) -> Sequence[TextEdit] | None:
     doc = ls.workspace.get_text_document(params.text_document.uri)
     lint_state = ls.lint_state(doc)
     return deb822_format_file(lint_state, _DEP5_FILE_METADATA)
@@ -184,7 +186,7 @@ def _debian_copyright_format_document(
 async def _debian_copyright_semantic_tokens_full(
     ls: "DebputyLanguageServer",
     request: SemanticTokensParams,
-) -> Optional[SemanticTokens]:
+) -> SemanticTokens | None:
     return await deb822_semantic_tokens_full(
         ls,
         request,
