@@ -1,9 +1,9 @@
 import re
 from typing import (
     Union,
-    Sequence,
     Optional,
 )
+from collections.abc import Sequence
 
 from debputy.linting.lint_util import LintState, with_range_in_continuous_parts
 from debputy.lsp.debputy_ls import DebputyLanguageServer
@@ -21,6 +21,7 @@ from debputy.lsp.lsp_features import (
     lsp_format_document,
     SecondaryLanguage,
     LanguageDispatchRule,
+    lsp_cli_reformat_document,
 )
 from debputy.lsp.lsp_generic_deb822 import (
     deb822_completer,
@@ -82,7 +83,7 @@ lsp_standard_handler(_DISPATCH_RULE, TEXT_DOCUMENT_CODE_ACTION)
 def debian_tests_control_hover(
     ls: "DebputyLanguageServer",
     params: HoverParams,
-) -> Optional[Hover]:
+) -> Hover | None:
     return deb822_hover(ls, params, _DTESTS_CTRL_FILE_METADATA)
 
 
@@ -90,7 +91,7 @@ def debian_tests_control_hover(
 def debian_tests_control_completions(
     ls: "DebputyLanguageServer",
     params: CompletionParams,
-) -> Optional[Union[CompletionList, Sequence[CompletionItem]]]:
+) -> CompletionList | Sequence[CompletionItem] | None:
     return deb822_completer(ls, params, _DTESTS_CTRL_FILE_METADATA)
 
 
@@ -98,7 +99,7 @@ def debian_tests_control_completions(
 def debian_tests_control_folding_ranges(
     ls: "DebputyLanguageServer",
     params: FoldingRangeParams,
-) -> Optional[Sequence[FoldingRange]]:
+) -> Sequence[FoldingRange] | None:
     return deb822_folding_ranges(ls, params, _DTESTS_CTRL_FILE_METADATA)
 
 
@@ -141,15 +142,16 @@ async def _lint_debian_tests_control(lint_state: LintState) -> None:
 def _debian_tests_control_on_save_formatting(
     ls: "DebputyLanguageServer",
     params: WillSaveTextDocumentParams,
-) -> Optional[Sequence[TextEdit]]:
+) -> Sequence[TextEdit] | None:
     doc = ls.workspace.get_text_document(params.text_document.uri)
     lint_state = ls.lint_state(doc)
     return deb822_format_file(lint_state, _DTESTS_CTRL_FILE_METADATA)
 
 
+@lsp_cli_reformat_document(_DISPATCH_RULE)
 def _reformat_debian_tests_control(
     lint_state: LintState,
-) -> Optional[Sequence[TextEdit]]:
+) -> Sequence[TextEdit] | None:
     return deb822_format_file(lint_state, _DTESTS_CTRL_FILE_METADATA)
 
 
@@ -157,7 +159,7 @@ def _reformat_debian_tests_control(
 def _debian_tests_control_on_save_formatting(
     ls: "DebputyLanguageServer",
     params: DocumentFormattingParams,
-) -> Optional[Sequence[TextEdit]]:
+) -> Sequence[TextEdit] | None:
     doc = ls.workspace.get_text_document(params.text_document.uri)
     lint_state = ls.lint_state(doc)
     return deb822_format_file(lint_state, _DTESTS_CTRL_FILE_METADATA)
@@ -167,7 +169,7 @@ def _debian_tests_control_on_save_formatting(
 async def _debian_tests_control_semantic_tokens_full(
     ls: "DebputyLanguageServer",
     request: SemanticTokensParams,
-) -> Optional[SemanticTokens]:
+) -> SemanticTokens | None:
     return await deb822_semantic_tokens_full(
         ls,
         request,
