@@ -1,13 +1,17 @@
 import functools
+import importlib.resources
 import itertools
 import os
 import re
 import subprocess
-from typing import Iterable, FrozenSet, Tuple, Optional, Container, TYPE_CHECKING
+from importlib.resources.abc import Traversable
+from typing import FrozenSet, Tuple, Optional, TYPE_CHECKING
+from collections.abc import Iterable, Container
 
 from debian.debian_support import Release
 from debputy.lsp.quickfixes import propose_correct_text_quick_fix
 from debputy.util import _info, _warn
+import debputy.lsp.data.wordlists as data_wordlist
 
 try:
     from debputy.lsp.vendoring._deb822_repro.locatable import (
@@ -103,9 +107,13 @@ except ImportError:
 
 
 def _read_wordlist(
-    base_dir: str, wordlist_name: str, *, namelist: bool = False
+    base_dir: Traversable,
+    wordlist_name: str,
+    *,
+    namelist: bool = False,
 ) -> Iterable[str]:
-    with open(os.path.join(base_dir, wordlist_name)) as fd:
+    path = base_dir.joinpath(wordlist_name)
+    with path.open("r", encoding="utf-8") as fd:
         w = [w.strip() for w in fd]
         yield from w
         if namelist:
@@ -123,16 +131,16 @@ def _all_debian_archs() -> Iterable[str]:
 
 
 @functools.lru_cache
-def _builtin_exception_words() -> FrozenSet[str]:
-    basedirs = os.path.dirname(__file__)
+def _builtin_exception_words() -> frozenset[str]:
     release_names = (x for x in Release.releases)
+    data_dir = importlib.resources.files(data_wordlist.__name__)
     return frozenset(
         itertools.chain(
             itertools.chain.from_iterable(
-                _read_wordlist(basedirs, wl) for wl in _WORDLISTS
+                _read_wordlist(data_dir, wl) for wl in _WORDLISTS
             ),
             itertools.chain.from_iterable(
-                _read_wordlist(basedirs, wl, namelist=True) for wl in _NAMELISTS
+                _read_wordlist(data_dir, wl, namelist=True) for wl in _NAMELISTS
             ),
             release_names,
             _all_debian_archs(),
@@ -182,7 +190,7 @@ def default_spellchecker() -> "Spellchecker":
     return spellchecker
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def _do_nothing_spellchecker() -> "Spellchecker":
     return EverythingIsCorrectSpellchecker()
 
@@ -201,7 +209,7 @@ def _testing_set_default_spellchecker(
     return previous
 
 
-def _skip_quoted_parts(line: str) -> Iterable[Tuple[str, int]]:
+def _skip_quoted_parts(line: str) -> Iterable[tuple[str, int]]:
     current_pos = 0
     while True:
         try:
@@ -226,7 +234,7 @@ def _skip_quoted_parts(line: str) -> Iterable[Tuple[str, int]]:
         current_pos = end_marker_pos + 1
 
 
-def _split_line_to_words(line: str) -> Iterable[Tuple[str, int, int]]:
+def _split_line_to_words(line: str) -> Iterable[tuple[str, int, int]]:
     for line_part, part_pos in _skip_quoted_parts(line):
         for m in _WORD_PARTS.finditer(line_part):
             fullword = m.group(1)
@@ -252,7 +260,7 @@ class Spellchecker:
     def do_nothing_spellchecker() -> "Spellchecker":
         return EverythingIsCorrectSpellchecker()
 
-    def iter_words(self, line: str) -> Iterable[Tuple[str, int, int]]:
+    def iter_words(self, line: str) -> Iterable[tuple[str, int, int]]:
         yield from _split_line_to_words(line)
 
     def provide_corrections_for(self, word: str) -> Iterable[str]:
@@ -272,7 +280,7 @@ class ContextIgnoredWordsSpellchecker(Spellchecker):
         self._spellchecker = spellchecker
         self._context_ignored_words = context_ignored_words
 
-    def iter_words(self, line: str) -> Iterable[Tuple[str, int, int]]:
+    def iter_words(self, line: str) -> Iterable[tuple[str, int, int]]:
         return self._spellchecker.iter_words(line)
 
     def provide_corrections_for(self, word: str) -> Iterable[str]:
