@@ -5,14 +5,12 @@ import os
 import re
 from enum import Enum
 from typing import (
-    Callable,
     Optional,
     TypeVar,
-    Iterable,
     Union,
-    Sequence,
     Tuple,
 )
+from collections.abc import Callable, Iterable, Sequence
 
 from debputy.intermediate_manifest import PathType
 from debputy.plugin.api import VirtualPath
@@ -24,7 +22,7 @@ MR = TypeVar("MR")
 _GLOB_PARTS = re.compile(r"[*?]|\[]?[^]]+]")
 
 
-def _lookup_path(fs_root: VP, path: str) -> Optional[VP]:
+def _lookup_path(fs_root: VP, path: str) -> VP | None:
     if not path.startswith("./"):
         raise ValueError("Directory must be normalized (and not the root directory)")
     if fs_root.name != "." or fs_root.parent_dir is not None:
@@ -35,7 +33,7 @@ def _lookup_path(fs_root: VP, path: str) -> Optional[VP]:
 
 def _compile_basename_glob(
     basename_glob: str,
-) -> Tuple[Optional[str], Callable[[str], bool]]:
+) -> tuple[str | None, Callable[[str], bool]]:
     remainder = None
     if not glob.has_magic(basename_glob):
         return escape_shell(basename_glob), lambda x: x == basename_glob
@@ -62,7 +60,7 @@ def _compile_basename_glob(
 
 def _apply_match(
     fs_path: VP,
-    match_part: Union[Callable[[str], bool], str],
+    match_part: Callable[[str], bool] | str,
 ) -> Iterable[VP]:
     if isinstance(match_part, str):
         m = fs_path.lookup(match_part)
@@ -95,7 +93,7 @@ class MatchRule:
         self,
         fs_root: VP,
         *,
-        ignore_paths: Optional[Callable[[VP], bool]] = None,
+        ignore_paths: Callable[[VP], bool] | None = None,
     ) -> Iterable[VP]:
         # TODO: Strictly speaking, this is unsound. (E.g., FSRootDir does not return FSRootDir on a lookup)
         raise NotImplementedError
@@ -104,7 +102,7 @@ class MatchRule:
         raise NotImplementedError
 
     @property
-    def path_type(self) -> Optional[PathType]:
+    def path_type(self) -> PathType | None:
         return None
 
     def describe_match_short(self) -> str:
@@ -121,7 +119,7 @@ class MatchRule:
         cls,
         directory: str,
         definition_source: str,
-        path_type: Optional[PathType] = None,
+        path_type: PathType | None = None,
         substitution: Substitution = NULL_SUBSTITUTION,
     ) -> "MatchRule":
         if directory in (".", "/"):
@@ -138,7 +136,7 @@ class MatchRule:
         cls,
         path_or_glob: str,
         definition_source: str,
-        path_type: Optional[PathType] = None,
+        path_type: PathType | None = None,
         substitution: Substitution = NULL_SUBSTITUTION,
     ) -> "MatchRule":
         # TODO: Handle '{a,b,c}' patterns too
@@ -230,7 +228,7 @@ class MatchAnything(MatchRule):
         return "**/*"
 
     def finditer(
-        self, fs_root: VP, *, ignore_paths: Optional[Callable[[VP], bool]] = None
+        self, fs_root: VP, *, ignore_paths: Callable[[VP], bool] | None = None
     ) -> Iterable[VP]:
         if ignore_paths is not None:
             yield from (p for p in fs_root.all_paths() if not ignore_paths(p))
@@ -256,7 +254,7 @@ class ExactFileSystemPath(MatchRule):
         return self._path
 
     def finditer(
-        self, fs_root: VP, *, ignore_paths: Optional[Callable[[VP], bool]] = None
+        self, fs_root: VP, *, ignore_paths: Callable[[VP], bool] | None = None
     ) -> Iterable[VP]:
         p = _lookup_path(fs_root, self._path)
         if p is not None and (ignore_paths is None or not ignore_paths(p)):
@@ -280,7 +278,7 @@ class DirectoryBasedMatch(MatchRule):
         self,
         rule_type: MatchRuleType,
         directory: str,
-        path_type: Optional[PathType] = None,
+        path_type: PathType | None = None,
     ) -> None:
         super().__init__(rule_type)
         self._directory = directory
@@ -298,7 +296,7 @@ class DirectoryBasedMatch(MatchRule):
         self,
         fs_root: VP,
         *,
-        ignore_paths: Optional[Callable[[VP], bool]] = None,
+        ignore_paths: Callable[[VP], bool] | None = None,
     ) -> Iterable[VP]:
         p = _lookup_path(fs_root, self._directory)
         if p is None or not p.is_dir:
@@ -330,7 +328,7 @@ class DirectoryBasedMatch(MatchRule):
         return f"{self.describe_match_short()} (anything directly in the directory)"
 
     @property
-    def path_type(self) -> Optional[PathType]:
+    def path_type(self) -> PathType | None:
         return self._path_type
 
     @property
@@ -356,9 +354,9 @@ class BasenameGlobMatch(MatchRule):
     def __init__(
         self,
         basename_glob: str,
-        only_when_in_directory: Optional[str] = None,
-        path_type: Optional[PathType] = None,
-        recursive_match: Optional[bool] = None,  # TODO: Can this just be = False (?)
+        only_when_in_directory: str | None = None,
+        path_type: PathType | None = None,
+        recursive_match: bool | None = None,  # TODO: Can this just be = False (?)
     ) -> None:
         super().__init__(MatchRuleType.BASENAME_GLOB)
         self._basename_glob = basename_glob
@@ -384,7 +382,7 @@ class BasenameGlobMatch(MatchRule):
         self,
         fs_root: VP,
         *,
-        ignore_paths: Optional[Callable[[VP], bool]] = None,
+        ignore_paths: Callable[[VP], bool] | None = None,
     ) -> Iterable[VP]:
         search_root = fs_root
         if self._directory is not None:
@@ -434,11 +432,11 @@ class BasenameGlobMatch(MatchRule):
         )
 
     @property
-    def path_type(self) -> Optional[PathType]:
+    def path_type(self) -> PathType | None:
         return self._path_type
 
     @property
-    def directory(self) -> Optional[str]:
+    def directory(self) -> str | None:
         return self._directory
 
     def shell_escape_pattern(self) -> str:
@@ -456,7 +454,7 @@ class GenericGlobImplementation(MatchRule):
     def __init__(
         self,
         glob_pattern: str,
-        path_type: Optional[PathType] = None,
+        path_type: PathType | None = None,
     ) -> None:
         super().__init__(MatchRuleType.GENERIC_GLOB)
         if glob_pattern.startswith("./"):
@@ -479,7 +477,7 @@ class GenericGlobImplementation(MatchRule):
         self,
         fs_root: VP,
         *,
-        ignore_paths: Optional[Callable[[VP], bool]] = None,
+        ignore_paths: Callable[[VP], bool] | None = None,
     ) -> Iterable[VP]:
         search_history = [fs_root]
         for part in self._match_parts:
@@ -523,7 +521,7 @@ class GenericGlobImplementation(MatchRule):
     def describe_match_exact(self) -> str:
         return f"{self.describe_match_short()} (glob)"
 
-    def _compile_glob(self) -> Sequence[Union[Callable[[str], bool], str]]:
+    def _compile_glob(self) -> Sequence[Callable[[str], bool] | str]:
         assert self._glob_pattern.strip("/") == self._glob_pattern
         return [
             _compile_basename_glob(part)[1] if glob.has_magic(part) else part
@@ -539,5 +537,5 @@ class GenericGlobImplementation(MatchRule):
         )
 
     @property
-    def path_type(self) -> Optional[PathType]:
+    def path_type(self) -> PathType | None:
         return self._path_type
