@@ -2,16 +2,14 @@ import dataclasses
 from typing import (
     TypeVar,
     TYPE_CHECKING,
-    Sequence,
     Tuple,
-    Mapping,
     Dict,
     Optional,
     TypedDict,
     NotRequired,
     List,
-    MutableMapping,
 )
+from collections.abc import Sequence, Mapping, MutableMapping
 
 if TYPE_CHECKING:
     from debputy.plugin.api import VirtualPath
@@ -25,17 +23,17 @@ else:
 
 
 class EnvironmentModificationSerialized(TypedDict):
-    replacements: NotRequired[Dict[str, str]]
-    removals: NotRequired[List[str]]
+    replacements: NotRequired[dict[str, str]]
+    removals: NotRequired[list[str]]
 
 
 @dataclasses.dataclass(slots=True, frozen=True)
 class EnvironmentModification:
-    replacements: Sequence[Tuple[str, str]] = tuple()
+    replacements: Sequence[tuple[str, str]] = tuple()
     removals: Sequence[str] = tuple()
 
     def __bool__(self) -> bool:
-        return not self.removals and not self.replacements
+        return bool(self.removals) or bool(self.replacements)
 
     def combine(
         self,
@@ -94,7 +92,7 @@ class EnvironmentModification:
             del env[k]
 
     def compute_env(self, base_env: Mapping[str, str]) -> Mapping[str, str]:
-        updated_env: Optional[Dict[str, str]] = None
+        updated_env: dict[str, str] | None = None
         for k, v in self.replacements:
             existing_value = base_env.get(k)
             if v == existing_value:
