@@ -1,3 +1,4 @@
+import logging
 from typing import TYPE_CHECKING
 
 import pytest
@@ -8,7 +9,7 @@ from debputy.architecture_support import DpkgArchitectureBuildProcessValuesTable
 from debputy.lsp.debputy_ls import DebputyLanguageServer
 from debputy.packages import DctrlParser
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
-from debputy.util import setup_logging
+from debputy.util import setup_logging, change_log_level
 
 if TYPE_CHECKING:
     import lsprotocol.types as types
@@ -27,6 +28,7 @@ except ImportError:
 @pytest.fixture(scope="session", autouse=True)
 def enable_logging() -> None:
     setup_logging(log_only_to_stderr=True, reconfigure_logging=True)
+    change_log_level(logging.INFO)
 
 
 @pytest.fixture
