@@ -1,6 +1,6 @@
 import stat
 import os
-from typing import Iterator
+from collections.abc import Iterator
 
 import pytest
 
