@@ -1,6 +1,7 @@
 import os
 import textwrap
-from typing import Sequence, Any, List, Optional
+from typing import List, Optional
+from collections.abc import Sequence
 
 import pytest
 
@@ -19,7 +20,7 @@ from debputy.plugin.api.test_api import (
 from debputy.plugin.api.test_api import manifest_variable_resolution_context
 from debputy.plugin.api.test_api.test_impl import initialize_plugin_under_test_preloaded
 from debputy.plugin.api.test_api.test_spec import DetectedService
-from debputy.plugin.debputy.build_system_rules import (
+from debputy.plugins.debputy.build_system_rules import (
     AutoconfBuildSystemRule,
     MakefileBuildSystemRule,
     PerlBuildBuildSystemRule,
@@ -29,10 +30,10 @@ from debputy.plugin.debputy.build_system_rules import (
     CMakeBuildSystemRule,
     MesonBuildSystemRule,
 )
-from debputy.plugin.debputy.debputy_plugin import initialize_debputy_features
-from debputy.plugin.debputy.private_api import load_libcap
-from debputy.plugin.debputy.service_management import SystemdServiceContext
-from debputy.plugin.debputy.types import DebputyCapability
+from debputy.plugins.debputy.debputy_plugin import initialize_debputy_features
+from debputy.plugins.debputy.private_api import load_libcap
+from debputy.plugins.debputy.service_management import SystemdServiceContext
+from debputy.plugins.debputy.types import DebputyCapability
 
 
 def test_debputy_packager_provided_files():
@@ -87,11 +88,13 @@ def test_debputy_packager_provided_files():
         "doc-base",
         "shlibs",
         "symbols",
+        "conffiles",
         "alternatives",
         "init",
         "default",
         "templates",
-        # dh_installsytemd
+        "udev",
+        # dh_installsystemd
         "mount",
         "path",
         "service",
@@ -306,7 +309,7 @@ def test_debputy_metadata_detector_xfonts():
     snippets = metadata.maintscripts()
     assert metadata.substvars["misc:Depends"] == "xfonts-utils"
     assert len(snippets) == 2
-    assert set(s.maintscript for s in snippets) == {"postinst", "postrm"}
+    assert {s.maintscript for s in snippets} == {"postinst", "postrm"}
     postinst_snippet = metadata.maintscripts(maintscript="postinst")[0]
     postrm_snippet = metadata.maintscripts(maintscript="postrm")[0]
 
@@ -359,7 +362,7 @@ def test_debputy_metadata_detector_xfonts_scale_and_alias():
     snippets = metadata.maintscripts()
     assert metadata.substvars["misc:Depends"] == "xfonts-utils"
     assert len(snippets) == 2
-    assert set(s.maintscript for s in snippets) == {"postinst", "postrm"}
+    assert {s.maintscript for s in snippets} == {"postinst", "postrm"}
     postinst_snippet = metadata.maintscripts(maintscript="postinst")[0]
     postrm_snippet = metadata.maintscripts(maintscript="postrm")[0]
 
@@ -418,7 +421,7 @@ def test_debputy_metadata_detector_icon_cache():
     metadata = plugin.run_metadata_detector(metadata_detector_id, fs_root)
     snippets = metadata.maintscripts()
     assert len(snippets) == 2
-    assert set(s.maintscript for s in snippets) == {"postinst", "postrm"}
+    assert {s.maintscript for s in snippets} == {"postinst", "postrm"}
     postinst_snippet = metadata.maintscripts(maintscript="postinst")[0]
     postrm_snippet = metadata.maintscripts(maintscript="postrm")[0]
 
@@ -470,7 +473,7 @@ def test_debputy_metadata_detector_kernel_modules():
     metadata = plugin.run_metadata_detector(metadata_detector_id, fs_root)
     snippets = metadata.maintscripts()
     assert len(snippets) == 4  # Two for each version
-    assert set(s.maintscript for s in snippets) == {"postinst", "postrm"}
+    assert {s.maintscript for s in snippets} == {"postinst", "postrm"}
     postinst_snippets = metadata.maintscripts(maintscript="postinst")
     postrm_snippets = metadata.maintscripts(maintscript="postrm")
 
@@ -583,7 +586,7 @@ def test_debputy_metadata_detector_pycompile_files():
     metadata = plugin.run_metadata_detector(metadata_detector_id, fs_root)
     snippets = metadata.maintscripts()
     assert len(snippets) == 2
-    assert set(s.maintscript for s in snippets) == {"postinst", "prerm"}
+    assert {s.maintscript for s in snippets} == {"postinst", "prerm"}
     postinst_snippets = metadata.maintscripts(maintscript="postinst")
     prerm_snippets = metadata.maintscripts(maintscript="prerm")
 
@@ -617,7 +620,7 @@ def test_debputy_metadata_detector_pycompile_files_private_package_dir():
     metadata = plugin.run_metadata_detector(metadata_detector_id, fs_root)
     snippets = metadata.maintscripts()
     assert len(snippets) == 2
-    assert set(s.maintscript for s in snippets) == {"postinst", "prerm"}
+    assert {s.maintscript for s in snippets} == {"postinst", "prerm"}
     postinst_snippets = metadata.maintscripts(maintscript="postinst")
     prerm_snippets = metadata.maintscripts(maintscript="prerm")
 
@@ -655,7 +658,7 @@ def test_system_service_detection() -> None:
     systemd_service_system_dir = f"{systemd_service_root_dir}/system"
     systemd_service_user_dir = f"{systemd_service_root_dir}/user"
 
-    services: List[DetectedService[Optional[object]]]
+    services: list[DetectedService[object | None]]
 
     services, _ = plugin.run_service_detection_and_integrations(
         "systemd", build_virtual_file_system([])
@@ -760,7 +763,7 @@ def test_sysv_service_detection() -> None:
     )
     init_dir = "etc/init.d"
 
-    services: List[DetectedService[Optional[object]]]
+    services: list[DetectedService[object | None]]
 
     services, _ = plugin.run_service_detection_and_integrations(
         "sysvinit", build_virtual_file_system([])
@@ -1285,7 +1288,7 @@ def test_auto_detect_autoconf_build_system(
     filename: str,
     expected: bool,
     mode: int,
-    content: Optional[str],
+    content: str | None,
 ) -> None:
     fs_root = build_virtual_file_system(
         [virtual_path_def(filename, mode=mode, content=content)]
