@@ -1,6 +1,7 @@
 import io
 import textwrap
-from typing import Callable, Optional, List, Tuple, Sequence
+from typing import Optional, List, Tuple
+from collections.abc import Callable, Sequence
 
 import pytest
 
@@ -25,6 +26,7 @@ from debputy.dh_migration.migrators_impl import (
     detect_obsolete_substvars,
     detect_dh_addons_zz_debputy_rrr,
     migrate_shell_completions,
+    migrate_clean_file,
 )
 from debputy.dh_migration.models import (
     FeatureMigration,
@@ -138,11 +140,11 @@ def _verify_migrator_generates_parsable_manifest(
     dh_config_name: str,
     dh_config_content: str,
     expected_manifest_content: str,
-    expected_warnings: Optional[List[str]] = None,
-    expected_renamed_paths: Optional[List[Tuple[str, str]]] = None,
-    expected_removals: Optional[List[str]] = None,
+    expected_warnings: list[str] | None = None,
+    expected_renamed_paths: list[tuple[str, str]] | None = None,
+    expected_removals: list[str] | None = None,
     required_plugins: Sequence[str] = tuple(),
-    dh_config_mode: Optional[int] = None,
+    dh_config_mode: int | None = None,
 ) -> None:
     # No file, no changes
     empty_fs = build_virtual_file_system([DEBIAN_DIR_ENTRY])
@@ -210,6 +212,74 @@ def _verify_migrator_generates_parsable_manifest(
     parser_factory().parse_manifest(fd=actual_manifest)
 
 
+def test_migrate_clean_no_content(
+    empty_manifest_pkg_foo: HighLevelManifest,
+    accept_no_migration_issues: AcceptableMigrationIssues,
+) -> None:
+    migrator = migrate_clean_file
+    empty_debian_dir = build_virtual_file_system([DEBIAN_DIR_ENTRY])
+    migration = run_migrator(
+        migrator,
+        empty_debian_dir,
+        empty_manifest_pkg_foo,
+        accept_no_migration_issues,
+    )
+
+    assert not migration.anything_to_do
+    assert not migration.warnings
+    assert not migration.remove_paths_on_success
+    assert not migration.rename_paths_on_success
+
+    empty_clean_file_debian_dir = build_virtual_file_system(
+        [
+            DEBIAN_DIR_ENTRY,
+            virtual_path_def(
+                "clean",
+                fs_path="/nowhere/debian/clean",
+                content="# Some comment",
+            ),
+        ]
+    )
+
+    migration = run_migrator(
+        migrator,
+        empty_clean_file_debian_dir,
+        empty_manifest_pkg_foo,
+        accept_no_migration_issues,
+    )
+
+    assert migration.anything_to_do
+    assert not migration.warnings
+    assert migration.remove_paths_on_success == ["/nowhere/debian/clean"]
+    assert not migration.rename_paths_on_success
+    assert not migration.successful_manifest_changes
+
+
+def test_migrate_clean_with_content(
+    manifest_parser_pkg_foo_factory: Callable[[], YAMLManifestParser],
+    accept_no_migration_issues: AcceptableMigrationIssues,
+) -> None:
+    migrator = migrate_clean_file
+
+    expected_manifest_content = textwrap.dedent(
+        """\
+            manifest-version: '0.1'
+            remove-during-clean:
+            - foo/*
+            - bar/
+    """
+    )
+
+    _verify_migrator_generates_parsable_manifest(
+        migrator,
+        manifest_parser_pkg_foo_factory,
+        accept_no_migration_issues,
+        "clean",
+        "foo/*\nbar/\n",
+        expected_manifest_content,
+    )
+
+
 def test_migrate_tmpfile(
     empty_manifest_pkg_foo: HighLevelManifest,
     accept_no_migration_issues: AcceptableMigrationIssues,
@@ -716,6 +786,7 @@ def test_migrate_install_file(
     dh_config_content = textwrap.dedent(
         """\
         bar usr/bin
+        ../foo.txt usr/share/foo/data
     """
     )
     expected_manifest_content = textwrap.dedent(
@@ -725,6 +796,9 @@ def test_migrate_install_file(
             - install:
                 source: bar
                 dest-dir: usr/bin
+            - install:
+                source: debian/foo.txt
+                dest-dir: usr/share/foo/data
     """
     )
     _verify_migrator_generates_parsable_manifest(
