@@ -1,7 +1,10 @@
-from typing import Mapping, Any, Optional
+from typing import Any, Optional, Union
+from collections.abc import Mapping
 
 import pytest
 from debian.deb822 import Deb822
+
+from debputy.plugin.api.test_api.test_impl import FAKE_DPKG_QUERY_TABLE
 from debputy.yaml.compat import CommentedMap
 
 from debputy.lsp.maint_prefs import (
@@ -52,6 +55,8 @@ def test_load_named_styles() -> None:
     # TODO: Not implemented yet
     assert not black_style.deb822_normalize_field_order
 
+    assert black_style.deb822_auto_canonical_size_field_names
+
 
 def test_compat_styles() -> None:
     styles = MaintainerPreferenceTable.load_preferences()
@@ -140,7 +145,18 @@ def test_x_style() -> None:
     assert tool == "debputy reformat"
 
 
-def test_was_from_salsa_ci_style() -> None:
+@pytest.mark.parametrize(
+    "variables",
+    [
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": "no"},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": 0},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": False},
+        {"SALSA_CI_ENABLE_WRAP_AND_SORT": "yes"},
+        {"SALSA_CI_ENABLE_WRAP_AND_SORT": 1},
+        {"SALSA_CI_ENABLE_WRAP_AND_SORT": True},
+    ],
+)
+def test_was_from_salsa_ci_style_enabled(variables: Mapping[str, Any]) -> None:
     styles = MaintainerPreferenceTable.load_preferences()
     fields = Deb822(
         {
@@ -153,22 +169,60 @@ def test_was_from_salsa_ci_style() -> None:
     effective_style, tool, _ = determine_effective_preference(styles, src, None)
     assert effective_style is None
     assert tool is None
-    salsa_ci = CommentedMap(
-        {"variables": CommentedMap({"SALSA_CI_DISABLE_WRAP_AND_SORT": "yes"})}
-    )
-    effective_style, tool, _ = determine_effective_preference(styles, src, salsa_ci)
-    assert effective_style is None
-    assert tool is None
 
-    salsa_ci = CommentedMap(
-        {"variables": CommentedMap({"SALSA_CI_DISABLE_WRAP_AND_SORT": "no"})}
-    )
+    salsa_ci = CommentedMap({"variables": CommentedMap(variables)})
     effective_style, tool, _ = determine_effective_preference(styles, src, salsa_ci)
     was_style = EffectiveFormattingPreference(**_WAS_DEFAULTS)
     assert effective_style == was_style
     assert tool == "wrap-and-sort"
 
 
+@pytest.mark.parametrize(
+    "variables",
+    [
+        {},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": "yes"},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": 1},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": True},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": {}},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": []},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": None},
+        {"SALSA_CI_ENABLE_WRAP_AND_SORT": "no"},
+        {"SALSA_CI_ENABLE_WRAP_AND_SORT": 0},
+        {"SALSA_CI_ENABLE_WRAP_AND_SORT": False},
+        {"SALSA_CI_ENABLE_WRAP_AND_SORT": {}},
+        {"SALSA_CI_ENABLE_WRAP_AND_SORT": []},
+        {"SALSA_CI_ENABLE_WRAP_AND_SORT": None},
+        {
+            "SALSA_CI_DISABLE_WRAP_AND_SORT": "yes",
+            "SALSA_CI_ENABLE_WRAP_AND_SORT": "yes",
+        },
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": 1, "SALSA_CI_ENABLE_WRAP_AND_SORT": 1},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": True, "SALSA_CI_ENABLE_WRAP_AND_SORT": True},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": {}, "SALSA_CI_ENABLE_WRAP_AND_SORT": True},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": [], "SALSA_CI_ENABLE_WRAP_AND_SORT": True},
+        {"SALSA_CI_DISABLE_WRAP_AND_SORT": None, "SALSA_CI_ENABLE_WRAP_AND_SORT": True},
+    ],
+)
+def test_was_from_salsa_ci_style_disabled(variables: Mapping[str, Any]) -> None:
+    styles = MaintainerPreferenceTable.load_preferences()
+    fields = Deb822(
+        {
+            "Package": "foo",
+            "Maintainer": "Random Developer <random@example.org>",
+        },
+    )
+    src = SourcePackage(fields)
+    assert "random@example.org" not in styles.maintainer_preferences
+    effective_style, tool, _ = determine_effective_preference(styles, src, None)
+    assert effective_style is None
+    assert tool is None
+    salsa_ci = CommentedMap({"variables": CommentedMap(variables)})
+    effective_style, tool, _ = determine_effective_preference(styles, src, salsa_ci)
+    assert effective_style is None
+    assert tool is None
+
+
 @pytest.mark.parametrize(
     "was_args,style_delta",
     [
@@ -212,7 +266,7 @@ def test_was_from_salsa_ci_style() -> None:
 )
 def test_was_from_salsa_ci_style_args(
     was_args: str,
-    style_delta: Optional[Mapping[str, Any]],
+    style_delta: Mapping[str, Any] | None,
 ) -> None:
     styles = MaintainerPreferenceTable.load_preferences()
     fields = Deb822(
