@@ -1,4 +1,5 @@
-from typing import Sequence, Union
+from typing import Union
+from collections.abc import Sequence
 
 import pytest
 
@@ -15,6 +16,6 @@ from debputy.util import escape_shell
         ("--foo with spaces=bar and baz", '"--foo with spaces=bar and baz"'),
     ],
 )
-def test_symlink_normalization(arg: Union[str, Sequence[str]], expected: str) -> None:
+def test_symlink_normalization(arg: str | Sequence[str], expected: str) -> None:
     actual = escape_shell(arg) if isinstance(arg, str) else escape_shell(*arg)
     assert actual == expected
